
#include "kdar.h"

#include "kdar_part/kdar_part.h"

#include <kkeydialog.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>

#include <kedittoolbar.h>

#include <kaction.h>
#include <kstdaction.h>

#include <kparts/mainwindow.h>
#include <kdockwidget.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <klibloader.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <klocale.h>

#include <qmultilineedit.h>

#include <iostream>

KDar::KDar()
    : KParts::MainWindow( 0L, "KDar" ),
      m_part( 0 )
{
    // set the shell's ui resource file
    setXMLFile( "kdar_shell.rc" );

    // this routine will find and load our Part.  it finds the Part by
    // name which is a bad idea usually.. but it's alright in this
    // case since our Part is made for this Shell
    KLibFactory *KDarPartFactory = KLibLoader::self()->factory( "libkdarpart" );
    if ( KDarPartFactory )
    {
        // now that the Part is loaded, we cast it to a Part to get
        // our hands on it
         m_part = static_cast<KDarPart *>( KDarPartFactory->create( this, "kdar_part", "KDarPart" ) );

        if ( m_part )
        {
            setCentralWidget( m_part->widget() );
            // then, setup our actions
            setupActions();
            //Disable building the standard help menu...
            setHelpMenuEnabled( false );
            //Instead, add a custom help menu to the menubar
            menuBar()->insertItem( i18n( "&Help" ), (QPopupMenu*) customHelpMenu() );
//            connect( m_help, SIGNAL( showAboutApplication() ), m_part, SLOT( showAboutDialog() ));
            // and a status bar
            statusBar()->show();
            // and integrate the part's GUI with the shell's
            createGUI( m_part );
            QPopupMenu *popup = static_cast<QPopupMenu*>( factory()->container( "settings", this ) );
            popup->insertItem( i18n( "&View" ), m_part->dockPopupMenu(), -1, 0 );
        }
    }
    else
    {
        // if we couldn't find our Part, we exit since the Shell by
        // itself can't do anything useful
        KMessageBox::error( this, i18n( "Could not find our part." ) );
        kapp->quit();
        // we return here, cause kapp->quit() only means "exit the
        // next time we enter the event loop...
        return;
    }


    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();
}

KDar::~KDar()
{
    std::cout << "KDar::~KDar() reached!" << std::endl;
}

void KDar::myquit()
{
    if ( m_part )
    {
        delete m_part;
        m_part = 0;
    }
    kapp->quit();
}

void KDar::load( const KURL& /*url*/ )
{
//    m_part->openURL( url );
}

void KDar::setupActions()
{
//     KStdAction::openNew( this, SLOT( fileNew() ), actionCollection() );
//     KStdAction::open( this, SLOT( fileOpen() ), actionCollection() );

    KStdAction::quit( this, SLOT( myquit() ), actionCollection() );

    m_toolbarAction = KStdAction::showToolbar( this, SLOT( optionsShowToolbar() ), actionCollection() );
    m_statusbarAction = KStdAction::showStatusbar( this, SLOT( optionsShowStatusbar() ), actionCollection() );

    KStdAction::keyBindings( this, SLOT( optionsConfigureKeys() ), actionCollection() );
    KStdAction::configureToolbars( this, SLOT( optionsConfigureToolbars() ), actionCollection() );

}

void KDar::saveProperties( KConfig* )
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored
}

void KDar::readProperties( KConfig* )
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
}

void KDar::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // About this function, the style guide (
    // http://developer.kde.org/documentation/standards/kde/style/basics/index.html )
    // says that it should open a new window if the document is _not_
    // in its initial state.  This is what we do here..
/*    if ( ! m_part->url().isEmpty() || m_part->isModified() )
    {
        (new KDar)->show();
    };
*/
}


void KDar::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if ( m_toolbarAction->isChecked() )
        toolBar()->show();
    else
        toolBar()->hide();
}

void KDar::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if ( m_statusbarAction->isChecked() )
        statusBar()->show();
    else
        statusBar()->hide();
}

void KDar::optionsConfigureKeys()
{
    KKeyDialog::configure( actionCollection() );
}

void KDar::optionsConfigureToolbars()
{
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    saveMainWindowSettings( KGlobal::config(), autoSaveGroup() );
# else
    saveMainWindowSettings( KGlobal::config() );
# endif
#else
    saveMainWindowSettings( KGlobal::config() );
#endif

    // use the standard toolbar editor
    KEditToolbar dlg( factory() );
    connect( &dlg, SIGNAL( newToolbarConfig() ), this, SLOT( applyNewToolbarConfig() ) );
    dlg.exec();
}

void KDar::applyNewToolbarConfig()
{
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    applyMainWindowSettings( KGlobal::config(), autoSaveGroup() );
# else
    applyMainWindowSettings( KGlobal::config() );
# endif
#else
    applyMainWindowSettings( KGlobal::config() );
#endif
}

void KDar::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
/*    KURL url =
        KFileDialog::getOpenURL( QString::null, QString::null, this );

    if ( url.isEmpty() == false )
    {
        // About this function, the style guide (
        // http://developer.kde.org/documentation/standards/kde/style/basics/index.html )
        // says that it should open a new window if the document is _not_
        // in its initial state.  This is what we do here..
        if ( m_part->url().isEmpty() && ! m_part->isModified() )
        {
            // we open the file in this window...
            load( url );
        }
        else
        {
            // we open the file in a new window...
            KDar* newWin = new KDar;
            newWin->load( url );
            newWin->show();
        }
    }
*/
}

void KDar::showAboutApplication()
{
    if ( m_part )
        m_part->showAboutDialog();
}

#include "kdar.moc"
