
#ifndef _KDAR_H_
#define _KDAR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kparts/dockmainwindow.h>
#include <kparts/mainwindow.h>

class KDarPart;
class KDockWidget;
class KHelpMenu;
class KToggleAction;

/**
 * This is the application "Shell".  It has a menubar, toolbar, and
 * statusbar but relies on the "Part" to do all the real work.
 *
 * @short Application Shell
 * @author Johnathan Burchill <jkerrb@shaw.ca>
 * @version 0.1
 */
class KDar : public KParts::MainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    KDar();

    /**
     * Default Destructor
     */
    virtual ~KDar();

    /**
     * Use this method to load whatever file/URL you have
     */
    void load(const KURL& url);

protected:
    /**
     * This method is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties( KConfig * );

    /**
     * This method is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readProperties( KConfig * );

protected slots:
    /**
     * We need to override this to show a custom about dialog.
     */
    virtual void showAboutApplication();

private slots:
    /**
     * Local cleanup
     */
    void myquit();
    void fileNew();
    void fileOpen();
    void optionsShowToolbar();
    void optionsShowStatusbar();
    void optionsConfigureKeys();
    void optionsConfigureToolbars();

    void applyNewToolbarConfig();

private:
    void setupAccel();
    void setupActions();

private:
    KDarPart* m_part;
    KDockWidget* browserDock;
    KDockWidget* managerDock;
    KDockWidget* messagesDock;
    KDockWidget* logDock;

    KToggleAction* m_toolbarAction;
    KToggleAction* m_statusbarAction;

};

#endif // _KDAR_H_
