/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdaropenarchivedialog.h"
#include "kdaropenarchivewidget.h"

#include "../../kcm_modules/kdarsettings.h"

#include <kfiledialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <kurlrequester.h>

#include <qcheckbox.h>
#include <qlayout.h>
#include <qpushbutton.h>

KDarOpenArchiveDialog::KDarOpenArchiveDialog( const QString & startDir, const QString & filter, QWidget* parent, const char* name, bool modal )
    : m_dialog( 0 ),
      m_options( 0 ),
      m_selectedFile( "" ),
      m_inputPipe( "" ),
      m_outputPipe( "" ),
      m_executeCommand( "" ),
      m_verbose( false )
{
    //prepare a button for selecting advanced opening options
    QWidget* optionsButton = new QWidget( this, "optionsToggleButton" );
    QHBoxLayout* box = new QHBoxLayout( optionsButton );
    QPushButton* advanced = new QPushButton( i18n( "Options" ), optionsButton, "Archive opening options" );
    advanced->setToggleButton( true );
    box->addWidget( advanced );
    QSpacerItem* spacer = new QSpacerItem( 30, 10, QSizePolicy::Expanding, QSizePolicy::Minimum );
    box->addItem( spacer );

    m_dialog = new KFileDialog( startDir, filter, parent, name, modal, optionsButton );
    m_dialog->setOrientation( Vertical );
    m_dialog->setOperationMode( KFileDialog::Opening );
    m_dialog->setMode( KFile::File | KFile::LocalOnly );
    m_options = new KDarOpenArchiveWidget( this );
    //prepare the options widget
    m_dialog->setExtension( m_options );
    connect( advanced, SIGNAL( toggled( bool ) ), m_dialog, SLOT( showExtension( bool ) ) );
}

KDarOpenArchiveDialog::~KDarOpenArchiveDialog()
{
    if ( m_dialog )
    {
        delete m_dialog;
        m_dialog = 0;
        // m_options is owned by the dialog now.
    }
}

int KDarOpenArchiveDialog::exec()
{
    int result = -1;
    if ( m_dialog )
    {
        result = m_dialog->exec();
        m_selectedFile = m_dialog->selectedFile();
        m_inputPipe = m_options->inputPipeKURLRequester->lineEdit()->text();
        m_outputPipe = m_options->outputPipeKURLRequester->lineEdit()->text();
        m_executeCommand = m_options->executeCommandKURLRequester->lineEdit()->text();
        m_verbose = m_options->verboseCheckBox->isChecked();
        return result;
    }
    else
    {
        return -1;
    }
}

#include "kdaropenarchivedialog.moc"
