/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KDARINTERACTION_H_
#define _KDARINTERACTION_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kdebug.h>

#include <string>

#include <dar/libdar.hpp>

class controller;
class libdar::statistics;


class KDarInteraction : public libdar::user_interaction
{
public:
    /**
     * Constructor
     *
     * @param[in] an optional pointer to the global controller.
     *            It has access to the password if the archive
     *            uses encryption.
     */
    KDarInteraction( controller *aController = 0 );
    /**
     * Destructor
     */
    virtual ~KDarInteraction();
    /**
     * Copy constructor
     */
    KDarInteraction( const KDarInteraction & );
    /**
     * Assignment operator
     */
    KDarInteraction & operator=( const KDarInteraction & );
    /**
     * Asks user a question and gives libdar the result
     * via an exception.
     */
    void pause( const std::string & );
    /**
     * Asks user for a response. Used for getting a password to encrypt or
     * decrypt an archive.
     *
     * @param echo indicates whether KDar should let the response be seen.
     *             This is useful for hiding passwords.
     */
    std::string get_string( const std::string & message, bool echo );
    /**
     * Presents a message to the user.
     */
    void inherited_warning( const std::string & message );
    /**
     * Clone method required by libdar.
     */
    libdar::user_interaction* clone() const;
    /**
     * Archive listings
     */
    void listing( const std::string & flag, const std::string & perm, const std::string & uid, const std::string & gid, const std::string & size, const std::string & date, const std::string & filename, bool is_dir, bool has_children );

    libdar::statistics* stats;

    void checkProgress();

private:
    /**
     * Sometimes we need to talk with a controller.
     */
    controller *m_controller;
};

#endif // _KDARINTERACTION_H_
