/***************************************************************************
 *   Copyright (C) 2004, 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "kdarthread.h"

#include "../controller.h"

#include "../events/kdardeletethreadevent.h"
#include "../events/kdararchivebrowserupdatesevent.h"
#include "../events/kdarcheckfilesizeevent.h"
#include "../events/kdarcancelbuttonevent.h"
#include "../events/kdarlogfilecommandevent.h"
#include "../events/kdarprogressevent.h"
#include "../events/kdarsetcursorevent.h"
#include "../events/kdarstatusbarevent.h"
#include "../events/kdarwarningevent.h"

#include <kdebug.h>
#include <klistview.h>
#include <klocale.h>

#include <qapplication.h>
#include <qevent.h>

KDarThread::KDarThread( controller * parent, const char* name )
    : QThread( 0 ),
      m_controller( parent ),
      m_name( name ),
      m_threadID( 0 )
{
    m_endMessage = QString::null;
}

KDarThread::~KDarThread()
{
    QApplication::sendPostedEvents( m_controller, KDar::LOGFILECOMMAND );
    logMessage( m_endMessage, true );
    closeLogFile();
}

void KDarThread::enableCancelButton( bool state )
{
    kdDebug() << m_name << "::enableCancelButton()" << endl;
    kdarCancelButtonEvent * cancelButton = new kdarCancelButtonEvent( state );
    QApplication::postEvent( m_controller, cancelButton );
}

void KDarThread::cancelOperation( bool urgent )
{
    kdDebug() << m_name << "::cancelOperation()" << endl;
    // m_threadID is the ID of the running thread. You need to make sure
    // that m_threadID is set in the "run" method of the derived class.
    if ( m_threadID )
    {
        libdar::cancel_thread( m_threadID, urgent );
    }
}

void KDarThread::setWaitCursor()
{
    kdDebug() << m_name << "::setWaitCursor()" << endl;
    kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor() );
    QApplication::postEvent( m_controller, setWaitCursor );
}

void KDarThread::setNormalCursor()
{
    kdDebug() << m_name << "::setNormalCursor()" << endl;
    kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor() );
    QApplication::postEvent( m_controller, setArrowCursor );
}

void KDarThread::statusMessage( const QString & message, const int duration )
{
    kdDebug() << m_name << "::statusMessage()" << endl;
    kdarStatusBarEvent *statusEvent = new kdarStatusBarEvent( message, duration );
    QApplication::postEvent( m_controller, statusEvent );

}

void KDarThread::logMessage( const QString & message, bool logFileOnly )
{
    kdDebug() << m_name << "::setLogMessage()" << endl;
    if ( message.isNull() ) return;
    if ( logFileOnly )
    {
        QCustomEvent *logEvent = new kdarLogFileCommandEvent( kdarLogFileCommandEvent::WRITEMESSAGE, message );
        QApplication::postEvent( m_controller, logEvent );
    }
    else
    {
        QCustomEvent *logEvent = new kdarWarningEvent( message );
        QApplication::postEvent( m_controller, logEvent );
    }
}

void KDarThread::openLogFile() const
{
    kdDebug() << m_name << "::openLogFile()" << endl;
    //open the log file
    std::cout << "KDarThread::openLogFile(): is this the right place for this?" << std::endl;
/*    kdDebug() << "KDarThread::openLogFile() creating new event." << endl;
    kdarLogFileCommandEvent * logCommandEvent = new kdarLogFileCommandEvent( kdarLogFileCommandEvent::OPEN, m_kdc->logFile() );
    kdDebug() << "KDarThread::openLogFile() posting the open logFile event" << endl;
    QApplication::postEvent( m_controller, logCommandEvent );
    kdDebug() << "KDarThread::openLogFile() done." << endl;
*/
}

void KDarThread::closeLogFile() const
{
    kdDebug() << m_name << "::closeLogFile()" << endl;
    //close the log file
    std::cout << "KDarThread::openLogFile(): is this the right place for this?" << std::endl;

/*    kdarLogFileCommandEvent * logCommandEvent = new kdarLogFileCommandEvent( kdarLogFileCommandEvent::CLOSE );
    QApplication::sendPostedEvents( m_controller, KDar::LOGFILECOMMAND );
    QApplication::postEvent( m_controller, logCommandEvent );
    kdDebug() << "KDarThread::closeLogFile()" << endl;
*/
}

void KDarThread::disableBrowser()
{
    kdDebug() << m_name << "::disableBrowser()" << endl;
    kdarArchiveBrowserUpdatesEvent * disableBrowser = new kdarArchiveBrowserUpdatesEvent( kdarArchiveBrowserUpdatesEvent::DISABLE );
    QApplication::postEvent( m_controller, disableBrowser );
}

void KDarThread::enableBrowser()
{
    kdDebug() << m_name << "::enableBrowser()" << endl;
    kdarArchiveBrowserUpdatesEvent * enableBrowser = new kdarArchiveBrowserUpdatesEvent( kdarArchiveBrowserUpdatesEvent::ENABLE );
    QApplication::postEvent( m_controller, enableBrowser );
}

void KDarThread::startProgressBarUpdates()
{
    kdDebug() << m_name << "::startProgressBarUpdates()" << endl;
    kdarProgressEvent *progressBarStart = new kdarProgressEvent( kdarProgressEvent::PROGRESS_START );
    QApplication::postEvent( m_controller, progressBarStart );
}

void KDarThread::stopProgressBarUpdates()
{
    kdDebug() << m_name << "::stopProgressBarUpdates()" << endl;
    kdarProgressEvent *progressBarStop = new kdarProgressEvent( kdarProgressEvent::PROGRESS_STOP );
    QApplication::postEvent( m_controller, progressBarStop );
}

void KDarThread::startCheckFileSizeUpdates()
{
    kdDebug() << m_name << "::startCheckFileSizeUpdates()" << endl;
    kdarCheckFileSizeEvent *checkFileSizeEventStart = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STARTTIMER );
    QApplication::postEvent( m_controller, checkFileSizeEventStart );
}

void KDarThread::stopCheckFileSizeUpdates()
{
    kdDebug() << m_name << "::stopCheckFileSizeUpdates()" << endl;
    kdarCheckFileSizeEvent *checkFileSizeEventStop = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STOPTIMER );
    QApplication::postEvent( m_controller, checkFileSizeEventStop );
}

void KDarThread::deleteMe()
{
    kdDebug() << m_name << "::deleteMe()" << endl;
    kdarDeleteThreadEvent * deleteThisThread = new kdarDeleteThreadEvent( this );
    QApplication::postEvent( m_controller, deleteThisThread );
}

QString KDarThread::timeDifference( const QDateTime & startTime, const QDateTime & endTime )
{
    kdDebug() << m_name << "::timeDifference()" << endl;
    QString timeString;
    timeString += i18n( "Time to do the libdar operation", "Total time: " );
    //calculate days, hours, minutes, seconds
    int days = startTime.daysTo( endTime );
    int seconds = startTime.secsTo( endTime );
    int totalTime = seconds;
    //days
    if ( days > 0 )
    {
        timeString += i18n( "%n day, ", "%n days, ", days );
        seconds -= days * 86400;
    }
    //hours
    int hours = seconds / 3600;
    if ( hours > 0 )
    {
        timeString += i18n( "%n hour, ", "%n hours, ", hours );
        seconds -= hours * 3600;
    }
    //minutes
    int minutes = seconds / 60;
    if ( minutes > 0 )
    {
        timeString += i18n( "%n minute, ", "%n minutes, ", minutes );
        seconds -= minutes * 60;
    }
    //seconds
    if ( seconds > 0 )
    {
        timeString += i18n( "%n second, ", "%n seconds, ", seconds );
    }
    //less than one second
    if ( totalTime == 0 )
    {
        int secs = startTime.secsTo( QDateTime( startTime.date() ) );
        endTime.addSecs( secs );
        timeString += i18n( "total time for operation, in milliseconds. Leave \", \" at the end.", "%1 seconds, " ).arg( endTime.toString( "0.zzz" ) );
    }
    timeString.remove( timeString.length() - 2, 2 );
    timeString += ".\n";
    //return string
    return timeString;
}
