/***************************************************************************
 *   Copyright (C) 2004, 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "openarchivethread.h"

#include "../kdarenums.h"
#include "../controller.h"
#include "../kdarinteraction.h"

#include "../archivehandle.h"
#include "../kdardeclarations.h"
// #include "kdarPostListViewItemEvent.h"
// #include "kdarSetCursorEvent.h"
// #include "kdarStatusBarEvent.h"

#include <kcursor.h>
#include <kdebug.h>
#include <klocale.h>

#include <qapplication.h>
#include <qstring.h>

#include <dar/erreurs.hpp>

#include <string>

OpenArchiveThread::OpenArchiveThread( controller * aController, const QString & filename, const QString & inputPipe, const QString & outputPipe, const QString & executeCommand, bool verbose, libdar::U_I blockSize )
    : KDarThread( aController ),
      m_filename( filename ),
      m_inputPipe( inputPipe ),
      m_outputPipe( outputPipe ),
      m_executeCommand( executeCommand ),
      m_verbose( verbose ),
      m_blockSize( blockSize )
{
}

OpenArchiveThread::~OpenArchiveThread()
{
}

void OpenArchiveThread::run()
{
    //Every class that is derived from KDarThread must set this member variable:
    m_threadID = pthread_self();
//    QString finishedMessage ( i18n( "Opening archive...done.\n" ) );
//    setWaitCursor();
    enableCancelButton( true );
//    statusMessage( i18n( "Opening archive...\n" ), FOREVER );

    //Need some way to interact with user:
    KDarInteraction userInteractionDialog( m_controller );
//    KDarInteraction userInteractionDialog;

    //check whether we have a valid archiveName:
    archiveHandle archiveHndl( m_filename );
    if ( archiveHndl.lastArchiveSliceNumber() == 0 )
    {
        deleteMe();
        return;
    }
    //Check the encryption type:
    libdar::crypto_algo cryptoAlgorithm = libdar::crypto_none;
    if ( m_controller->handlesStrongCrypto() )
    {
        //This should be performed by libdar. It's in the libdar TODO...
        cryptoAlgorithm = archiveHndl.queryCryptoAlgorithm();
    }
    try
    {
        //We can keep many archives open at once. No need to close an archive.
/*        if ( m_controller )
        {
            libdar::archive* controllerArchive = m_controller->archive();
            if ( controllerArchive )
            {
                delete controllerArchive;
                controllerArchive = 0;
                m_controller->setArchive( controllerArchive );
            }
        }*/
        libdar::archive* arch = new libdar::archive( userInteractionDialog, archiveHndl.libdarPath(), archiveHndl.basename(), EXTENSION, cryptoAlgorithm, "", m_blockSize, m_inputPipe, m_outputPipe, m_executeCommand, m_verbose );
        if ( arch )
        {
            m_controller->setArchive( arch, m_filename );
        }
        arch->get_children_of( userInteractionDialog, "" );
    }
    catch( libdar::Elibcall &e )
    {
        kdDebug() << "openArchiveThread::run(): " << e.get_message() << endl;
    }
    catch( libdar::Euser_abort &e )
    {
        kdDebug() << "openArchiveThread::run(): user cancelled operation." << endl;
//        finishedMessage = i18n( "Opening archive...cancelled by user." );
//        logMessage( finishedMessage );
//        if ( theArchive )
//        {
//            delete theArchive;
//            theArchive = 0;
//        }
    }
    catch( libdar::Egeneric &e )
    {
        kdDebug() << "openArchiveThread::run(): " << e.get_message() << endl;
//         userInteractionDialog.warning( e.get_message() );
//         if ( theArchive )
//         {
//             delete theArchive;
//             theArchive = 0;
//         }
// //         finishedMessage = i18n( "Error: unable to open the archive.\n" );
//         logMessage( finishedMessage );
    }
    catch( ... )
    {
        kdDebug() << "openarchivethread::run(): unknown exception!" << endl;
    }
//     setNormalCursor();
    enableCancelButton( false );
//     statusMessage( finishedMessage );
    deleteMe();
}

