/***************************************************************************
 *   Copyright (C) 2004, 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "testarchivethread.h"

#include "../controller.h"
#include "../kdardeclarations.h"
#include "../kdarinteraction.h"
#include "../kdarstatistics.h"
#include "../events/kdarsetcursorevent.h"
#include "../events/kdarstatusbarevent.h"
#include "../events/kdarwarningevent.h"

#include <kapplication.h>
#include <kcursor.h>
#include <kdebug.h>
#include <klocale.h>

#include <qdatetime.h>
#include <qstring.h>

#include <dar/libdar.hpp>
#include <dar/tools.hpp>
#include <dar/erreurs.hpp>

#include <string>

TestArchiveThread::TestArchiveThread( controller * aController, libdar::archive* archive, bool verbose, bool showSkipped )
    : KDarThread( aController, "TestArchiveThread" ),
      m_archive( archive ),
      m_verbose( verbose ),
      m_showSkipped( showSkipped )
{}

TestArchiveThread::~TestArchiveThread(){}

void TestArchiveThread::run()
{
    //Every class that is derived from KDarThread must set this member variable:
    m_threadID = pthread_self();

    //Start timing the operation:
    QDateTime startTime = QDateTime::currentDateTime();
    openLogFile();
    setWaitCursor();
    enableCancelButton( true );
/*    logMessage( "\n" + i18n( "BEGIN TEST: " ) + startTime.toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_controller->archive + "\n", true );*/
/*    logMessage( i18n( "command line: " ) + m_darCommandLine + "\n", true );*/
    statusMessage( i18n( "Testing archive...\n" ), FOREVER );

    //Need some way to interact with user:
    KDarInteraction ki( m_controller );

    //file masks
    //Directories to include or exclude
    libdar::et_mask subtree;
//    libdar::et_mask subtree = m_kdc->libdarCreateDirectoryMask();
    //Also check for selected item in the archive file browser:
//     libdar::ou_mask subIncludes = buildDirectoryMaskFromListView( "<ROOT>/" );
//     if ( subIncludes.size() > 0 )
//     {
//         subtree.add_mask( subIncludes );
//         kdDebug() << "testing got subIncludes." << endl;
//     }
//     else
//     {
//         subtree.add_mask( libdar::bool_mask( true ) );
//     }

//    subtree.add_mask(libdar::bool_mask(true));
    try
    {
        kdDebug() << m_name << "::run(): starting test." << endl;
        if ( m_archive )
        {
            libdar::statistics st = m_archive->op_test( ki, libdar::bool_mask( true ), libdar::bool_mask( true ), true, false, ki.stats );
            kdDebug() << m_name << "::run(): finished test." << endl;
            KDarStatistics kstats( st );
        }
/*        logMessage( ks.testStatistics( (KDar::logLevel) m_kdc->logLevel() ) );
        statusMessage( i18n( "Testing archive...done.\n" ) );
        logMessage( timeDifference( startTime, QDateTime::currentDateTime() ) );*/
    }
    catch( libdar::Euser_abort &e )
    {
        QString errorMessage = i18n( "Testing archive...cancelled by user." );
        statusMessage( errorMessage );
        logMessage( errorMessage );
    }
    catch( libdar::Egeneric &e )
    {
        ki.warning( e.get_message() );
        QString errorMessage = i18n( "Testing archive...ERROR: could not test archive." );
        statusMessage( errorMessage );
        logMessage( e.get_message().c_str() );
        logMessage( errorMessage );
    }
    setNormalCursor();
    enableCancelButton( false );
    statusMessage( i18n( "Finished testing archive." ), MESSAGEDURATION );
    /*    m_endMessage = i18n( "END TEST: " ) + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_archiveName + "\n";*/
    deleteMe();
}

