/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarcompressionconfigdialog.h"

#include "kdarcombobox.h"

#include <klistview.h>
#include <knuminput.h>

#include <qhbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>

#include <iostream>

KDarCompressionConfigDialog::KDarCompressionConfigDialog( QWidget* parent, const char* name )
    : KDarFilterConfigDialog( parent, name )
{
    KDarFilterConfigLayout->setSpacing( 8 );
    //Insert the "type" label and a kdarcombobox
    QHBox* hbox1 = new QHBox( this, "hbox1" );
    hbox1->setSpacing( 5 );
    (void ) new QLabel( "Type", hbox1, "label1" );
    kcfg_CompressionType = new KDarComboBox( hbox1, "kcfg_CompressionType" );
    kcfg_CompressionType->insertItem( "none" );
    kcfg_CompressionType->insertItem( "gzip" );
    kcfg_CompressionType->insertItem( "bzip2" );
    connect( kcfg_CompressionType, SIGNAL( itemChanged( int ) ), this, SLOT( slotSetEnabledState( int ) ) );
    connect( kcfg_CompressionType, SIGNAL( activated( int ) ), this, SLOT( slotSetEnabledState( int ) ) );
    QLabel* hspacer1 = new QLabel( hbox1, "hspacer1" );
    hspacer1->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
    KDarFilterConfigLayout->insertWidget( 0, hbox1 );

    //Insert the "compression level" label and a kintspinbox
    QHBox* hbox2 = new QHBox( this, "hbox2" );
    hbox2->setSpacing( 5 );
    m_levelLabel = new QLabel( "gzip compression level", hbox2, "levelLabel" );
    m_levelLabel->setIndent( 32 );
    kcfg_CompressionLevel = new KIntSpinBox( hbox2, "kcfg_CompressionLevel" );
    kcfg_CompressionLevel->setMinValue( 1 );
    kcfg_CompressionLevel->setMaxValue( 9 );
    QLabel* hspacer2 = new QLabel( hbox2, "hspacer2" );
    hspacer2->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
    KDarFilterConfigLayout->insertWidget( 1, hbox2 );
    kcfg_CompressionLevel->setEnabled( false );
    m_levelLabel->setEnabled( false );

    //Insert the "compression level" label and a kintspinbox
    QHBox* hbox3 = new QHBox( this, "hbox3" );
    hbox3->setSpacing( 5 );
    m_sizeLabel = new QLabel( "Minimum size for compression", hbox3, "minimumsizelabel" );
    kcfg_MinimumCompressionSize = new KIntSpinBox( hbox3, "kcfg_MinimumCompressionSize" );
    kcfg_MinimumCompressionSize->setMinValue( 0 );
    kcfg_MinimumCompressionSize->setMaxValue( 1024 );
    kcfg_MinimumCompressionSize->setSuffix( " B" );
    kcfg_MinimumCompressionSize->setValue( 150 );
    QLabel* hspacer3 = new QLabel( hbox3, "hspacer3" );
    hspacer3->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
    KDarFilterConfigLayout->insertWidget( 2, hbox3 );

    kcfg_CompressionType->setCurrentItem( 2 );
}

KDarCompressionConfigDialog::~KDarCompressionConfigDialog()
{
}

void KDarCompressionConfigDialog::slotSetEnabledState( int level )
{
    bool showCompressionOptions = true;
    if ( level == 0 )
        showCompressionOptions = false;
    m_levelLabel->setEnabled( showCompressionOptions );
    kcfg_CompressionLevel->setEnabled( showCompressionOptions );
    m_sizeLabel->setEnabled( showCompressionOptions );
    kcfg_MinimumCompressionSize->setEnabled( showCompressionOptions );
    filterIncludePixmap->setEnabled( showCompressionOptions );
    filterExcludePixmap->setEnabled( showCompressionOptions );
    includeLabel->setEnabled( showCompressionOptions );
    excludeLabel->setEnabled( showCompressionOptions );
    filters->setEnabled( showCompressionOptions );
    filterAdd->setEnabled( showCompressionOptions );
    filterRemove->setEnabled( showCompressionOptions );
    filterMoveUp->setEnabled( showCompressionOptions );
    filterMoveDown->setEnabled( showCompressionOptions );
    filterToggleType->setEnabled( showCompressionOptions );

    bool showLevel = false;
    if ( level == 1 )
        showLevel = true;
    kcfg_CompressionLevel->setEnabled( showLevel );
    m_levelLabel->setEnabled( showLevel );

}

#include "kdarcompressionconfigdialog.moc"
