/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarconfig.h"
#include "kdarcompareconfigdialog.h"
#include "kdarcompressionconfigdialog.h"
#include "kdarcryptographyconfigdialog.h"
#include "kdarfilesystemconfigdialog.h"
#include "kdarfilterconfigdialog.h"
#include "kdarfilteritem.h"
#include "kdarfontconfigdialog.h"
#include "kdargeneralconfigdialog.h"
#include "kdarrestoreconfigdialog.h"
#include "kdarsliceconfigdialog.h"
#include "kdarsettings.h"

#include <kcombobox.h>
#include <kgenericfactory.h>
#include <kiconloader.h>

#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qwidget.h>

#include <iostream>

KDarConfig::KDarConfig( QWidget* parent, const char* name, const QStringList & )
    : KCModule( parent, name )
{
//    QVBoxLayout *topLayout = new QVBoxLayout( this );
    m_generalDlg = new KDarGeneralConfigDialog( this, "General" );
    m_fontDlg = new KDarFontConfigDialog( this, "Fonts" );
    m_sliceDlg = new KDarSliceConfigDialog( this, "Slices" );
    m_compressionDlg = new KDarCompressionConfigDialog( this, "Compression" );
    m_cryptographyDlg = new KDarCryptographyConfigDialog( this, "Cryptography" );
    m_directoryDlg = new KDarFilterConfigDialog( this, "Directory" );
    m_fileDlg = new KDarFilterConfigDialog( this, "File" );
    m_filesystemDlg = new KDarFilesystemConfigDialog( this, "Filesystem" );
    m_restoreDlg = new KDarRestoreConfigDialog( this, "Restore" );
    m_compareDlg = new KDarCompareConfigDialog( this, "Compare" );
//    topLayout->addWidget( m_dialog );

    setupModules();
}

KDarConfig::~KDarConfig()
{
    if ( m_generalDlg ) delete m_generalDlg;
    if ( m_fontDlg ) delete m_fontDlg;
    if ( m_sliceDlg ) delete m_sliceDlg;
    if ( m_compressionDlg ) delete m_compressionDlg;
    if ( m_cryptographyDlg ) delete m_cryptographyDlg;
    if ( m_directoryDlg ) delete m_directoryDlg;
    if ( m_fileDlg ) delete m_fileDlg;
    if ( m_filesystemDlg ) delete m_filesystemDlg;
    if ( m_restoreDlg ) delete m_restoreDlg;
    if ( m_compareDlg ) delete m_compareDlg;

}

QWidget* KDarConfig::generalDialog() const
{
    return (QWidget*) m_generalDlg;
}

QWidget* KDarConfig::fontDialog() const
{
    return (QWidget*) m_fontDlg;
}

QWidget* KDarConfig::sliceDialog() const
{
    return (QWidget*) m_sliceDlg;
}

QWidget* KDarConfig::compressionDialog() const
{
    return (QWidget*) m_compressionDlg;
}

QWidget* KDarConfig::cryptographyDialog() const
{
    return (QWidget*) m_cryptographyDlg;
}

QWidget* KDarConfig::directoryDialog() const
{
    return (QWidget*) m_directoryDlg;
}

QWidget* KDarConfig::fileDialog() const
{
    return (QWidget*) m_fileDlg;
}

QWidget* KDarConfig::filesystemDialog() const
{
    return (QWidget*) m_filesystemDlg;
}

QWidget* KDarConfig::restoreDialog() const
{
    return (QWidget*) m_restoreDlg;
}

QWidget* KDarConfig::compareDialog() const
{
    return (QWidget*) m_compareDlg;
}

void KDarConfig::load()
{
    std::cout << "KDarConfig::load() reached" << std::endl;
}

void KDarConfig::save()
{
    std::cout << "KDarConfig::save() reached" << std::endl;
}

void KDarConfig::defaults()
{
    std::cout << "KDarConfig::defaults() reached" << std::endl;
}

void KDarConfig::setupModules()
{
    //Slice module
    //Set the media type combo box entries
    QStringList mediaTypes;
    mediaTypes << "Custom";
    mediaTypes << "DVD (4.7 GB)";
    mediaTypes << "DVD (2.0 GB)";
    mediaTypes << "Zip (100 MB)";
    mediaTypes << "CD-R (80 min.)";
    mediaTypes << "CD-R (74 min.)";
    mediaTypes << "Floppy (1440 kB)";
    m_sliceDlg->kcfg_MediaType->clear();
    m_sliceDlg->kcfg_MediaType->insertStringList( mediaTypes );
    //connect the kcfg_MediaType activated(int) signal to our slot
    connect( m_sliceDlg->kcfg_MediaType, SIGNAL( activated( int ) ), this, SLOT( slotMediaChanged( int ) ) );
    //file filters
    connect( m_fileDlg, SIGNAL( widgetChanged() ), this, SIGNAL( widgetChanged() ) );
}

QString KDarConfig::sliceSize()
{
    return m_sliceDlg->kcfg_SliceSize->text();
}

int KDarConfig::sliceSizeSuffix()
{
    return m_sliceDlg->kcfg_SliceSizeSuffix->currentItem();
}

QString KDarConfig::firstSliceSize()
{
    return m_sliceDlg->kcfg_FirstSliceSize->text();
}

int KDarConfig::firstSliceSizeSuffix()
{
    return m_sliceDlg->kcfg_FirstSliceSizeSuffix->currentItem();
}

int KDarConfig::mediaType()
{
    return m_sliceDlg->kcfg_MediaType->currentItem();
}

void KDarConfig::slotMediaChanged( int newMedia )
{
    std::cout << "KDarConfig::slotMediaChanged() reached." << std::endl;
    m_sliceDlg->kcfg_SliceSize->setText( ( KDarSettings::mediaTypeSizes() )[ newMedia ] );
    m_sliceDlg->kcfg_SliceSizeSuffix->setCurrentItem( (KDarSettings::mediaTypeSizeSuffices())[ newMedia ] );
    m_sliceDlg->kcfg_FirstSliceSize->setText( (KDarSettings::mediaTypeFirstSizes())[ newMedia ] );
    m_sliceDlg->kcfg_FirstSliceSizeSuffix->setCurrentItem( (KDarSettings::mediaTypeFirstSizeSuffices())[ newMedia ] );
}

extern "C"
{
    KDE_EXPORT KCModule *create_kdarconfig( QWidget *parent, const char *name )
    {
        return new KDarConfig( parent, name );
    };
}

#include "kdarconfig.moc"
