/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarfilterconfigdialog.h"
#include "kdarfilteritem.h"

#include <kiconloader.h>
#include <klistview.h>
#include <klocale.h>

#include <qlabel.h>
#include <qpixmap.h>
#include <qptrlist.h>
#include <qpushbutton.h>

#include <iostream>

KDarFilterConfigDialog::KDarFilterConfigDialog( QWidget* parent, const char* name )
    : KDarFilterConfig( parent, name )
{
    filterIncludePixmap->setPixmap( SmallIcon( "button_ok" ) );
    filterExcludePixmap->setPixmap( SmallIcon( "button_cancel" ) );

    //Disable sorting: filter order matters
    filters->setSorting( -1 );

    //Connect the filter buttons to some slots
    connect( filterAdd, SIGNAL( clicked() ), this, SLOT( slotFilterItemAdd() ) );
    connect( filterRemove, SIGNAL( clicked() ), this, SLOT( slotFilterItemRemove() ) );
    connect( filterMoveUp, SIGNAL( clicked() ), this, SLOT( slotFilterItemMoveUp() ) );
    connect( filterMoveDown, SIGNAL( clicked() ), this, SLOT( slotFilterItemMoveDown() ) );
    connect( filterToggleType, SIGNAL( clicked() ), this, SLOT( slotFilterItemToggleType() ) );
}

KDarFilterConfigDialog::~KDarFilterConfigDialog()
{
}

void KDarFilterConfigDialog::addFilter( const QString & label, const int type, const bool automatic, const bool afterCurrent, const bool startEdit )
{
    KDarFilterItem* tmpItem = 0;
    if ( afterCurrent )
        tmpItem = (KDarFilterItem*) filters->currentItem();
    if ( !tmpItem )
        tmpItem = (KDarFilterItem*) filters->lastItem();
    //TODO add check to prevent user from adding duplicate filter?
    KDarFilterItem* newItem = new KDarFilterItem( filters, tmpItem, label, type, automatic );
    if ( startEdit )
        newItem->startRename( 0 );
//    std::cout << "KDarFilterConfigDialog::slotFilterItemAdd(): need to implement." << std::endl;
}

void KDarFilterConfigDialog::setFilters( const QStringList & filterList )
{
    //Clear the listview:
    filters->clear();
    QString label;
    int type;
    bool automatic;
    QString sep( "," );

    for ( QStringList::ConstIterator it = filterList.begin(); it != filterList.end(); ++it )
    {
        //Parse the item string
        label = (*it).section( sep, 0, 0 );
        type = (*it).section( sep, 1, 1 ).toInt();
        automatic = ( bool ) (*it).section( sep, 2, 2 ).toInt();
        //Add the filter to the listView...
        addFilter( label, type, automatic, false, false );
    }
}

QStringList KDarFilterConfigDialog::getFilters() const
{
    QStringList filterList;
    QString sep( "," );
    //We construct a QString for each filter, and add each filter to a list:
    KDarFilterItem* item = (KDarFilterItem*) filters->firstChild();
    while ( item )
    {
        filterList << item->text( 0 ) + sep + QString::number( item->type() ) + sep + QString::number( (int) item->isAutomatic() );
        item = (KDarFilterItem*) item->nextSibling();
    }
    return filterList;
}

void KDarFilterConfigDialog::slotFilterItemAdd()
{
    //Maybe show a custom dialog here, then add the item with the user-selected
    //settings...?
    addFilter();
    emit widgetChanged();
}

void KDarFilterConfigDialog::slotFilterItemRemove()
{
    QListViewItem* item;
    QPtrList<QListViewItem> list = filters->selectedItems();
    for ( item = list.first(); item; item = list.next() )
        delete item;
    emit widgetChanged();
}

void KDarFilterConfigDialog::slotFilterItemMoveUp()
{
    QListViewItem* item;
    QPtrList<QListViewItem> list = filters->selectedItems();
    for ( item = list.first(); item; item = list.next() )
    {
        //This is so complicated because itemAbout() ignores disabled items!
        QListViewItem* filterItem = filters->firstChild();
        QListViewItem* previousItem = 0;
        QListViewItem* previousPreviousItem = 0;
        while ( filterItem )
        {
            if ( filterItem == item && previousItem && !previousItem->isSelected() )
            {
                filters->moveItem( item, 0, previousPreviousItem );
                break;
            }
            previousPreviousItem = previousItem;
            previousItem = filterItem;
            filterItem = filterItem->nextSibling();
        }
    }
    emit widgetChanged();
}

void KDarFilterConfigDialog::slotFilterItemMoveDown()
{
    QListViewItem* item;
    QPtrList<QListViewItem> list = filters->selectedItems();
    for ( item = list.last(); item; item = list.prev() )
    {
        QListViewItem* nextItem = item->nextSibling();
        if ( nextItem && !nextItem->isSelected() )
            filters->moveItem( item, 0, nextItem );
    }
    emit widgetChanged();
}

void KDarFilterConfigDialog::slotFilterItemToggleType()
{
    QListViewItem* item;
    QPtrList<QListViewItem> list = filters->selectedItems();
    for ( item = list.first(); item; item = list.next() )
    {
        KDarFilterItem* filterItem = dynamic_cast<KDarFilterItem*>(item);
        if ( filterItem )
            filterItem->toggleType();
    }
    emit widgetChanged();
}

#include "kdarfilterconfigdialog.moc"
