/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "../../config.h"
#include "kdar_part.h"
#include "kdardockwidgets/kdaractionview.h"
#include "kdardockwidgets/kdarbrowser.h"
#include "kdardockwidgets/kdarmanager.h"
#include "kdardockwidgets/kdarmessages.h"
#include "kdardockwidgets/kdarlog.h"

//controller
#include "controllers/controller.h"
#include "controllers/kdarcontrollerfactory.h"
#include "controllers/archivehandle.h"

//archive manager
#include "kdarmanager/archivemanager.h"
#include "kdarmanager/kdarmanageritem.h"

//The configuration dialog
#include "kcm_modules/kdarconfigdialog.h"
#include "kcm_modules/kdarsettings.h"

//"About" dialog
#include "misc_widgets/kdaraboutdialog.h"

//The "Add Archive" widget

#include "misc_widgets/kdaraddarchivedialog.h"

//KDE includes
#include <kaboutdata.h>
#include <kaction.h>
#include <kapplication.h>
#include <kconfigdialog.h>
#include <kdebug.h>
#include <kdockwidget.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kinstance.h>
#include <klibloader.h>
#include <klistview.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kprinter.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <ktoolbar.h>

//QT includes
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <qvaluelist.h>

//System includes
#include <iostream>

KDarPart::KDarPart( QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name )
    : KParts::ReadOnlyPart( parent, name ),
      m_controller( 0 ),
      m_archiveManager( 0 ),
      m_printer( 0 ),
      m_aboutDialog( 0 )
{
    // we need an instance
    setInstance( KDarPartFactory::instance() );

    m_dockWindow = new KDockArea( parentWidget, "KDockArea" );
    //Create the dock windows: Actions, Browser, Manager, and Messages
    KDockWidget* actionsDock = m_dockWindow->createDockWidget( "Actions", SmallIcon( "kdar_actions" ), 0L, i18n( "KDar Actions" ) );
    KDockWidget* browserDock = m_dockWindow->createDockWidget( "Browser", SmallIcon( "kdar" ), 0L, i18n( "KDar Browser" ) );
    KDockWidget* managerDock = m_dockWindow->createDockWidget( "Manager", SmallIcon( "kdar_extract" ), 0L, i18n( "KDar Manager" ) );
    KDockWidget* messagesDock = m_dockWindow->createDockWidget( "Messages", SmallIcon( "kdar_view" ), 0L, i18n( "KDar Messages" ) );
    KDockWidget* logDock = m_dockWindow->createDockWidget( "Log", SmallIcon( "kdar_view" ), 0L, i18n( "KDar Log" ) );

    // set our XML-UI resource file
    setXMLFile( "kdar_part.rc" );
    // KDar actions
    m_actions = new KDarActionView( parentWidget, "KDar Actions" );
//     m_actions = new KDarActionView( parentWidget, widgetName );
    // the archive browser
    m_browser = new KDarBrowser( parentWidget, "KDar Browser" );
    // the manager widget
    m_manager = new KDarManager( parentWidget, "KDar Manager" );
    // the message window widget
    m_messages = new KDarMessages( parentWidget, "KDar Messages" );
    // the log window widget
    //For now, KDar won't provide a log viewer. The log is for trouble-shooting
    //and record-keeping.
//     m_log = new KDarLog( parentWidget, "KDar Log" );


    //Add the browser dock widget:
    browserDock->setWidget( m_browser );
    browserDock->setDockSite( KDockWidget::DockFullDocking );
    m_dockWindow->setMainDockWidget( browserDock );

    //Add the actions dock widget:
    actionsDock->setWidget( m_actions );
    actionsDock->manualDock( browserDock, KDockWidget::DockLeft, 30 );

    //Add the manager dock widget:
    managerDock->setWidget( m_manager );
//     managerDock->setDockSite( KDockWidget::DockFullDocking );
    managerDock->manualDock( actionsDock, KDockWidget::DockBottom, 35 );

    //Add the message window dock widget:
    messagesDock->setWidget( m_messages );
//     messagesDock->setDockSite( KDockWidget::DockFullDocking );
    messagesDock->manualDock( browserDock, KDockWidget::DockBottom, 70 );
    m_dockWindow->makeDockInvisible( messagesDock );

    //Add the log window dock widget as a tab to the messages window:
//     logDock->setWidget( m_log );
// //     logDock->setDockSite( KDockWidget::DockFullDocking );
//     logDock->manualDock( messagesDock, KDockWidget::DockRight );
//     m_dockWindow->makeDockInvisible( logDock );

    // notify the part that "m_dockWindow" is our internal widget
    setWidget( m_dockWindow );

    // create our actions
    setupActions();
    //restore dock window status
    restoreDockStatus();

    //Get a controller...
    getController();
    if ( m_controller )
    {
        m_browser->setController( m_controller );
        connect( m_browser, SIGNAL( childrenRequested( QString & ) ), m_controller, SLOT( slotGetChildren( QString & ) ) );
        connect( m_controller, SIGNAL( openedArchive( const KURL & ) ), this, SLOT( addRecentArchive( const KURL & ) ) );
    }
    //Set up archive manager
    m_archiveManager = new ArchiveManager( m_manager );
//    m_messages->append( "Messages are shown here..." );
//    m_log->append( "This is the log window..." );
}

KDarPart::~KDarPart()
{
    m_recentFilesAction->saveEntries( KGlobal::config(), "General" );
    kdDebug() << "KDarPart::~KDarPart(): saving dock window status" << endl;
    saveDockStatus();
    if ( m_controller ) delete m_controller;
}

void KDarPart::saveDockStatus()
{
    m_dockWindow->manager()->writeConfig( kapp->config(), "Docking Config" );
}

void KDarPart::restoreDockStatus()
{
    m_dockWindow->manager()->readConfig( kapp->config(), "Docking Config" );
}

QWidget* KDarPart::actionsDockWidget()
{
    return (QWidget*) m_actions;
}

QWidget* KDarPart::browserDockWidget()
{
    return (QWidget*) m_browser;
}

QWidget* KDarPart::managerDockWidget()
{
    return (QWidget*) m_manager;
}

QWidget* KDarPart::messagesDockWidget()
{
    return (QWidget*) m_messages;
}

QWidget* KDarPart::logDockWidget()
{
    return (QWidget*) m_log;
}

void KDarPart::setCancelState( bool state )
{
    if ( m_cancel )
        m_cancel->setEnabled( state );
}

void KDarPart::appendMessage( const QString & message )
{
    if ( !message.isNull() && m_messages )
    {
        m_messages->append( message );
    }
}

void KDarPart::setStatusBarMessage( const QString & message, const int duration )
{
    if ( !message.isNull() )
    {
        emit setStatusBarText( message );
    }
    if ( duration > 0 )
    {
        QTimer::singleShot( duration, this, SLOT( clearStatusBarMessages() ) );
    }
}

void KDarPart::getController()
{
    KLibFactory *controllerFactory = KLibLoader::self()->factory( "libkdarcontroller" );
    if ( controllerFactory )
    {
        m_controller = static_cast<controller*>( controllerFactory->create( this, "kdar_part" ) );
    }
}

void KDarPart::openArchive( const KURL & archiveName )
{
    if ( m_controller && !archiveName.isEmpty() )
    {
        m_controller->openArchive( archiveName.path() );
    }
}

bool KDarPart::openFile()
{
    kdDebug() << "KDarPart::openFile(): m_file is " << m_file << endl;
    if ( m_controller && !m_file.isEmpty() )
    {
        //show only the browser?
        m_dockWindow->makeDockInvisible( m_dockWindow->manager()->getDockWidgetFromName( "Manager" ) );
        m_dockWindow->makeDockInvisible( m_dockWindow->manager()->getDockWidgetFromName( "Actions" ) );
        m_dockWindow->makeDockInvisible( m_dockWindow->manager()->getDockWidgetFromName( "Log" ) );

        m_controller->openArchive( m_file, true );
        return true;
    }
    else
    {
        return false;
    }

}

void KDarPart::clearStatusBarMessages()
{
    emit setStatusBarText( "" );
}

void KDarPart::showAboutDialog()
{
    if ( !m_aboutDialog )
        m_aboutDialog = new KDarAboutDialog( widget(), "aboutkdar", i18n( "The KDE Disk archiver" ), VERSION, "Johnathan Burchill", "2004,2005", instance()->aboutData(), m_controller );

    if ( m_aboutDialog )
        m_aboutDialog->show();
}

void KDarPart::setupActions()
{
    //set up the actions
    (void) new KAction( i18n( "Print from &Browser..." ), "printer1", 0, this, SLOT( printBrowser() ), actionCollection(), "print_browser" );
    (void) new KAction( i18n( "Print from &Manager..." ), "printer1", 0, this, SLOT( printManager() ), actionCollection(), "print_manager" );
    (void) new KAction( i18n( "Print Me&ssages..." ), "printer1", 0, this, SLOT( printMessages() ), actionCollection(), "print_messages" );
    (void) new KAction( i18n( "Print &Log..." ), "printer1", 0, this, SLOT( printLog() ), actionCollection(), "print_log" );

    //Toolbar, statusbar.
//    m_toolbarAction = KStdAction::showToolbar( this, SLOT( optionsShowToolbar() ), actionCollection() );
//    m_statusbarAction = KStdAction::showStatusbar( this, SLOT( optionsShowStatusbar() ), actionCollection() );
    m_recentFilesAction = KStdAction::openRecent( this, SLOT( openArchive( const KURL & ) ), actionCollection(), "open_recent" );
    m_recentFilesAction->loadEntries( KGlobal::config(), "General" );

//    KStdAction::keyBindings( this, SLOT( optionsConfigureKeys() ), actionCollection() );
//    KStdAction::configureToolbars( this, SLOT( optionsConfigureToolbars() ), actionCollection() );
    KStdAction::preferences( this, SLOT( optionsPreferences() ), actionCollection() );

    (void) new KAction( i18n( "&Open archive" ), "kdar", 0, this, SLOT( chooseArchive() ), actionCollection(), "select_archive" );
    (void) new KAction( i18n( "&View" ), "kdar_view", 0, this, SLOT( listArchive() ), actionCollection(), "list_archive" );
    (void) new KAction( i18n( "&Create archive..." ), "kdar_create", 0, this, SLOT( createArchive() ), actionCollection(), "create_archive" );
    KAction *kdarRestore( new KAction( i18n( "&Restore files..." ), "kdar_extract", 0, this, SLOT( restoreArchive() ), actionCollection(), "extract_archive" ) );
    (void) new KAction( i18n( "&Isolate catalog..." ), "kdar_isolate", 0, this, SLOT( isolateArchive() ), actionCollection(), "isolate_archive" );
    KAction *kdarDiff( new KAction(i18n( "&Compare with filesystem..." ), "kdar_diff", 0, this, SLOT( diffArchive() ), actionCollection(), "diff_archive" ) );
    KAction *kdarTest( new KAction( i18n( "Chec&k integrity..." ), "kdar_test", 0, this, SLOT( testArchive() ), actionCollection(), "test_archive" ) );

    m_showPreviewsAction = new KToggleAction( i18n( "&Show Previews..." ), "view_icon", 0, actionCollection(), "show_previews" );
    connect( m_showPreviewsAction, SIGNAL( toggled( bool ) ), m_browser, SLOT( togglePreviews( bool ) ) );
    m_showPreviewsAction->setChecked( KDarSettings::showBrowserPreviews() );

    (void) new KAction( i18n( "Clear &Message Window" ), "eraser", 0, this, SLOT( clearMessages()), actionCollection(), "clear_messages" );
    (void) new KAction( i18n( "Clear &Log Window" ), "eraser", 0, this, SLOT( clearLog()), actionCollection(), "clear_log" );
    m_cancel = new KAction( i18n( "&Stop operation" ), "stop", 0, this, SLOT( cancelOperation() ), actionCollection(), "stop_operation" );
    m_cancel->setEnabled( false );
    //Main actions
    (void) ( new KAction(i18n( "&Full backup" ), "kdar_diff", 0, this, SLOT( fullBackup() ), actionCollection(), "full_backup" ) );
    (void) ( new KAction(i18n( "&Differential backup" ), "kdar_diff", 0, this, SLOT( diffBackup() ), actionCollection(), "diff_backup" ) );
    (void) ( new KAction(i18n( "Full &restore" ), "kdar_restore", 0, this, SLOT( fullRestore() ), actionCollection(), "full_restore" ) );
    //Manager actions
    KAction *managerNewDatabaseAction( new KAction( i18n( "&New database" ), "kdar_database", 0, this, SLOT( managerNewDatabase() ), actionCollection(), "new_database" ) );
    KAction *managerAddArchiveAction( new KAction( i18n( "&Add archive" ), "kdar_add", 0, this, SLOT( managerAddArchive() ), actionCollection(), "add_archive" ) );
    KAction *managerRemoveArchiveAction( new KAction( i18n( "&Remove archive" ), "kdar_remove", 0, this, SLOT( managerRemoveArchive() ), actionCollection(), "remove_archive" ) );
    KAction *managerSearchArchivesAction( new KAction( i18n( "&Search managed archives" ), "kdar_search", 0, this, SLOT( managerSearchManagedArchives() ), actionCollection(), "search_managed_archives" ) );
    //popup menus
    //Browser
    browserPopupMenu = new KActionMenu( "Archive browser", this );
    browserPopupMenu->insert( kdarRestore );
    browserPopupMenu->insert( kdarTest );
    browserPopupMenu->insert( m_showPreviewsAction );
    //Manager
    managerPopupMenu = new KActionMenu( "Archive manager", this );
    managerPopupMenu->insert( managerNewDatabaseAction );
    managerPopupMenu->insert( managerAddArchiveAction );
    managerPopupMenu->insert( managerRemoveArchiveAction );
    //connect the listview signals to the popups
    connect( m_browser, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint&) ), this, SLOT( showBrowserPopupMenu( KListView*, QListViewItem*, const QPoint& ) ) );
    connect( m_manager, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint&) ), this, SLOT( showManagerPopupMenu( KListView*, QListViewItem*, const QPoint& ) ) );
}

QPopupMenu* KDarPart::dockPopupMenu()
{
    return m_dockWindow->dockHideShowMenu();
}

void KDarPart::showBrowserPopupMenu( KListView*, QListViewItem*, const QPoint & point )
{
    browserPopupMenu->popup( point );
}

void KDarPart::showManagerPopupMenu( KListView*, QListViewItem*, const QPoint & point )
{
    managerPopupMenu->popup( point );
}

void KDarPart::printBrowser()
{
    std::cout << "KDarPart::printBrowser(): need to implement" << std::endl;
}

void KDarPart::printManager()
{
    if ( m_manager )
    {
        std::cout << "KDarPart::printManager(): info " << std::endl;
        if ( m_messages )
        {
            QPtrList<QListViewItem> selectedItems = m_manager->selectedItems();
            while ( selectedItems.current() )
            {
                KDarManagerItem* infoItem = static_cast<KDarManagerItem*>( selectedItems.current() );
                if ( m_archiveManager )
                    m_messages->append( m_manager->tooltip( infoItem ) );
                selectedItems.next();
            }
        }
    }
    std::cout << "KDarPart::printManager(): need to implement" << std::endl;
}

void KDarPart::printMessages()
{
    if ( !m_printer ) m_printer = new KPrinter;
    if ( m_printer->setup( m_messages ) )
        m_messages->print( m_printer );
}

void KDarPart::printLog()
{
    if ( !m_printer ) m_printer = new KPrinter;
    if ( m_printer->setup( m_log ) )
        m_log->print( m_printer );
}

void KDarPart::optionsPreferences()
{

    if ( KDarConfigDialog::showDialog( "settings" ) )
        return;

    //KConfigDialog didn't find an instance of this dialog, so lets create it :
    KDarConfigDialog* dialog = new KDarConfigDialog( widget() );

    if ( dialog->isValid() )
        dialog->show();
    else
    {
        std::cout << "KDarPart::optionsPreferences(): could not construct a" << std::endl;
        std::cout << "   configuration dialog. This probably means that the" << std::endl;
        std::cout << "   kcm_kdar config library could not be loaded." << std::endl;
        std::cout << "   Check your KDar installation." << std::endl;
        //Dialog is not valid, so delete it
        delete dialog;
    }
}

void KDarPart::readConfig()
{
/*
    m_verboseOutput = KDarSettings::verboseOutput();
    m_useCreateArchiveWizard = KDarGeneralSettings::useCreateArchiveWizard();
*/
}

void KDarPart::saveSettings()
{
/*
    //General
    KDarSettings::setVerboseOutput( m_verboseOutput );
    KDarSettings::setUseCreateArchiveWizard( m_useCreateArchiveWizard );
    //Fonts
    KDarSettings::setVerboseOutput( m_verboseOutput );
    KDarSettings::setUseCreateArchiveWizard( m_useCreateArchiveWizard );
*/
}

void KDarPart::filePrint()
{
    std::cout << "KDarPart::filePrint(): need to implement" << std::endl;
}

void KDarPart::clearMessages()
{
    m_messages->clear();
}

void KDarPart::clearLog()
{
    m_log->clear();
}

void KDarPart::cancelOperation()
{
    std::cout << "KDarPart::cancelOperation(): need to implement" << std::endl;
}

void KDarPart::fullBackup()
{
    std::cout << "KDarPart::fullBackup(): need to implement" << std::endl;
}

void KDarPart::diffBackup()
{
    std::cout << "KDarPart::diffBackup(): need to implement" << std::endl;
}

void KDarPart::fullRestore()
{
    std::cout << "KDarPart::fullRestore(): need to implement" << std::endl;
}

void KDarPart::managerNewDatabase()
{
    std::cout << "KDarPart::managerNewDatabase(): need to implement" << std::endl;
    if ( m_archiveManager )
    {
        m_archiveManager->addDatabase( "/opt/user/backups/dar_backups/manager/testing" );
    }
}

void KDarPart::managerAddArchive()
{
    std::cout << "KDarPart::managerAddArchive(): need to implement" << std::endl;
    if ( m_archiveManager )
    {
        //Show dialog asking user to select an archive to add to the manager
        KDarAddArchiveDialog* dialog = new KDarAddArchiveDialog( widget(), i18n( "Add archive to manager" ) );
        //Load the dialog with the list of managed archives...
        dialog->clearReferenceList();
        QListViewItemIterator it( m_manager );
        while ( it.current() )
        {
            KDarManagerItem* item = (KDarManagerItem*) it.current();
            if ( item )
            {
                std::cout << "KDarPart::managerAddArchive(): adding " << item->name() << std::endl;
                dialog->addReferenceArchive( item );
            }
            ++it;
        }
        if ( dialog->exec() )
        {
            //Add the archive to the manager
            QString name = dialog->name();
            QString root = dialog->root();
            QString path = dialog->path();
            QString status( "unknown" );
            QString notes = dialog->notes();
            KListViewItem* parent = dialog->referenceArchive();
            if ( parent )
                std::cout << "KDarPart::managerAddArchive(): reference archive is " << parent->text( 0 ) << std::endl;
            if ( !name.isEmpty() && !path.isEmpty() )
            {
                if ( dialog->isDifferential() && parent )
                {
                    QListViewItemIterator it( m_manager );
                    while ( it.current() )
                    {
                        KDarManagerItem* item = static_cast<KDarManagerItem*>( it.current() );
                        if ( item->name() == parent->text( 0 ) )
                        {
                            m_archiveManager->addArchive( name, root, path, item, status, notes );
                            break;
                        }
                        ++it;
                    }
                }
                else
                    m_archiveManager->addArchive( name, root, path, status, notes );
            }
        }
        if ( dialog )
            delete dialog;
    }
}

void KDarPart::managerRemoveArchive()
{
    //get selected items
    if ( m_manager )
    {
        QListViewItem* item;
        QPtrList<QListViewItem> selectedItems = m_manager->selectedItems();
        for ( item = selectedItems.first(); item; item = selectedItems.next() )
            m_archiveManager->removeArchive( (KDarManagerItem*) item, false );
        //Save the new list of managed archives:
        m_archiveManager->saveManagedArchives();
    }
}

void KDarPart::managerSearchManagedArchives()
{
    std::cout << "KDarPart::managerSearchManagedArchives(): need to implement" << std::endl;
}

void KDarPart::addRecentArchive( const KURL & archiveName )
{
    if ( !archiveName.isEmpty() )
    {
        m_recentFilesAction->addURL( archiveName );
    }
}

void KDarPart::removeRecentArchive( const KURL & archiveName )
{
    if ( !archiveName.isEmpty() )
    {
        m_recentFilesAction->removeURL( archiveName );
    }
}

void KDarPart::chooseArchive()
{
    if ( m_controller )
    {
        m_controller->openArchive();
    }
    else
        std::cout << "KDarPart::chooseArchive(): ERROR! no controller!" << std::endl;
}

void KDarPart::listArchive()
{
    std::cout << "KDarPart::listArchive(): who called this? I'm deprecated!" << std::endl;
/*    if ( m_controller )
        m_controller->listArchive( true );
    else
        std::cout << "KDarPart::listArchive(): ERROR! no controller!" << std::endl;*/
}

void KDarPart::createArchive()
{
    if ( m_controller )
        m_controller->createArchive();
    else
        std::cout << "KDarPart::createArchive(): ERROR! no controller!" << std::endl;
}

void KDarPart::restoreArchive()
{
    if ( m_controller )
        m_controller->restoreArchive();
    else
        std::cout << "KDarPart::restoreArchive(): ERROR! no controller!" << std::endl;
}

void KDarPart::isolateArchive()
{
    if ( m_controller )
        m_controller->isolateArchive();
    else
        std::cout << "KDarPart::isolateArchive(): ERROR! no controller!" << std::endl;
}

void KDarPart::diffArchive()
{
    if ( m_controller )
        m_controller->diffArchive();
    else
        std::cout << "KDarPart::diffArchive(): ERROR! no controller!" << std::endl;
}

void KDarPart::testArchive()
{
    if ( m_controller )
        m_controller->testArchive();
    else
        std::cout << "KDarPart::testArchive(): ERROR! no controller!" << std::endl;
}



// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance*  KDarPartFactory::s_instance = 0L;
KAboutData* KDarPartFactory::s_about = 0L;

KDarPartFactory::KDarPartFactory()
    : KParts::Factory()
{
}

KDarPartFactory::~KDarPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KDarPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name, const char * /*classname*/, const QStringList &/*args*/ )
{
    // Create an instance of our Part
    KDarPart* obj = new KDarPart( parentWidget, widgetName, parent, name );

    return obj;
}

KInstance* KDarPartFactory::instance()
{
    if( !s_instance )
    {
        s_about = new KAboutData( "kdarpart", I18N_NOOP( "KDarPart" ), VERSION, 0, KAboutData::License_GPL );
        s_about->setShortDescription( I18N_NOOP( "A filesystem backup and restore program for KDE." ) );
        s_about->setHomepage( "http://kdar.sourceforge.net" );
        s_about->setBugAddress( "http://sourceforge.net/tracker/?group_id=96947&atid=616472" );
        s_about->addAuthor( "Johnathan Burchill", I18N_NOOP( "Maintainer and developer." ), "jkerrb@users.sourceforge.net" );
        s_about->addCredit( "Denis Corbin", I18N_NOOP("Author and maintainer of libdar."), 0, "http://sourceforge.net/projects/dar");
        s_about->addCredit( "Mark McSweeny", I18N_NOOP( "For an in-depth tutorial on C++ programming techniques and pitfalls. Also for suggesting the polling method for cancelling libdar operations, among other things. Cheers." ), 0, 0);
        s_about->addCredit( "Sven Bergner", I18N_NOOP("Code cleanups in kdar-1.0-beta1"), 0, 0);
        s_about->addCredit( "Simon Batchelor, Walter Gray, Andreas Pour, Christos Francheteau, Aad Rijnberg", I18N_NOOP("For reporting compilation problems/fixes."), 0, 0);
        s_about->addCredit( "WWagner", I18N_NOOP("For reporting directory include/exclude problem."), 0, 0);
        s_about->addCredit( "Alban Minassian", I18N_NOOP("For reporting directory filter bug, along with fix. Also for suggesting that a double-click on the archive name should list the contents in the file browser."), 0, 0);
        s_about->addCredit( "Eugenia Loli-Queru", I18N_NOOP("For reporting the crash due to missing directory in new archive paths. Also for suggesting the default backup directory idea."), 0, 0);
        s_about->addCredit( "Daniel Nelson", I18N_NOOP( "For reporting the missing directory filters bug (#995601)" ), 0, 0 );
        s_about->addCredit( "Andrew E. Schulman", I18N_NOOP( "For reporting the unsaved execute command bug (#995780), the endless-loop bug (#995772), the missing archive name bug (#996539), and many others!. Also for suggesting many features." ), 0, 0 );
        s_about->addCredit( "Axel Braun", I18N_NOOP( "For reporting the unexpected archive problem when directory filters are not subdirectories of the root (bug#1061114)." ), 0, 0);
        s_about->addCredit( "Peter Landgren", I18N_NOOP( "For reporting the \"//\" crash (bug#1071164), and the total time display error (bug#1076005)." ), 0, 0);
        s_about->addCredit( "Hans-Ulrich Steck", I18N_NOOP( "For reporting the problem where filters were not set when loading a new profile (bug#1044696)." ), 0, 0);
        s_about->addCredit( "Gour", I18N_NOOP( "For reporting the Gentoo 64-bit configure problem (bug#1075570)." ), 0, 0);
        s_about->addCredit( "nathanv", I18N_NOOP( "For reporting the individual file restore bug#1088069)." ), 0, 0);
        s_about->addCredit( "Jeffrey Ratcliffe", I18N_NOOP( "For reporting the export command bugs #1097255 and #1097257." ), 0, 0);
        s_about->addCredit( "Anders Lund", I18N_NOOP( "For several suggestions concerning kdar profiles. Also for finding the problem with the default storage directory set to \"/\". (bug#1102120)" ), 0, 0);
        s_about->addCredit( "Mark Richards", I18N_NOOP( "For reporting the export command missing basename bug#1098706." ), 0, 0);

        s_instance = new KInstance( s_about );
    }
    return s_instance;
}

extern "C"
{
    void* init_libkdarpart()
    {
	KGlobal::locale()->insertCatalogue( "kdar" );
        return new KDarPartFactory;
    }
};

#include "kdar_part.moc"
