/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill   *
 *   jkerrb@shaw.ca   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDARPARTKDARBROWSER_H
#define KDARPARTKDARBROWSER_H

#include <qwidget.h>
#include <qhbox.h>

#include <klistviewsearchline.h>
#include <kurl.h>

class kdarListViewItem;
class controller;
class KActionMenu;
class KDarBrowserSearchWidget;
class KDarBrowserSearchLine;
class KImageFilePreview;
class KListView;
class KListViewSearchLine;
class KListViewSearchLineWidget;
class KToggleAction;

class QCheckBox;
class QListViewItem;
class QPoint;
class QPopupMenu;
class QString;

#include <dar/libdar.hpp>

/**
@author Johnathan Burchill
*/
class KDarBrowser : public QWidget
{
    Q_OBJECT
public:
    KDarBrowser( QWidget* parent = 0, const char * name = 0 );

    virtual ~KDarBrowser();

    void setController( controller* aController ) { m_controller = aController; }

    virtual void addBrowserEntry( QString filename, QString permissions, QString uid, QString gid, QString size, QString date, QString status, bool isDirectory, bool hasChildren );

    virtual void clearBrowserView();
    virtual void addArchive( libdar::archive* arch, const QString & path, const QString & displayedName );

signals:
    void childrenRequested( QString & parent );
    void contextMenu( KListView*, QListViewItem*, const QPoint& );

public slots:
    void togglePreviews( bool show );

protected slots:
    void slotItemExpanded( QListViewItem* );
    void slotContextMenu( KListView*, QListViewItem*, const QPoint& );
    void showPreview( QListViewItem* );
    void setShownArchive( QListViewItem* item, const QPoint &, int );

private:
    void removePreviewURL( const KURL & url );

private:
    //The actual listview
    KListView* m_browserView;
    //A preview widget
    KImageFilePreview* m_preview;

    //This listview is for keeping track of opened archives
    KListView* m_archiveView;
    //The controller
    controller* m_controller;

    //A search widget
    KDarBrowserSearchWidget* m_searchWidget;
//     KListViewSearchLineWidget* m_searchWidget;
    //Stores the current item for populating with children
    kdarListViewItem* m_currentItem;

    //Keeps track of previously previewed URL so it can be removed from
    //disk when a new preview is requested.
    KURL m_previousURL;
};

/**
 * Derived from KListViewSearchLineWidget. This one enables QRegExp.
 */
class KDarBrowserSearchWidget : public QHBox
{
public:
    KDarBrowserSearchWidget( KListView* lv = 0, QWidget* parent = 0, const char* name = 0 );

    virtual ~KDarBrowserSearchWidget() { if ( m_searchline ) delete m_searchline; }

private:
    KListViewSearchLine* m_searchline;
//     QCheckBox* m_globCheckBox;
};

/**
 * Derived from KListViewSearchLine. This one enables QRegExp.
 */
class KDarBrowserSearchLine : public KListViewSearchLine
{
    Q_OBJECT
public:
    KDarBrowserSearchLine( QWidget* parent = 0, KListView* listview = 0, const char* name = 0 );
    virtual ~KDarBrowserSearchLine();
    /**
     * This checks for matches using regular expressions.
     */
    bool itemMatches( const QListViewItem* item, const QString & s ) const;

public slots:
    void setGlobbing( bool use );

protected:
    QPopupMenu* createPopupMenu();

private:
    bool m_usingGlobs;
    KToggleAction* m_useGlobsAction;
    KActionMenu* menu;
};
#endif
