/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@sourceforge.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdarlog.h"

#include <kdebug.h>
#include <klocale.h>
#include <kprinter.h>
#include <ktextedit.h>

#include <qlayout.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qrect.h>
#include <qsimplerichtext.h>
#include <qstylesheet.h>

KDarLog::KDarLog( QWidget* parent, const char * name )
    : QWidget( parent, name )
{
    QGridLayout* logLayout = new QGridLayout( this );
    logLayout->setAutoAdd( true );
    //Set up the KListView:
    m_log = new KTextEdit( this, "KDarLog" );
    m_log->setReadOnly( true );
}


KDarLog::~KDarLog()
{
}

void KDarLog::append( const QString & message )
{
    if ( !message.isNull() ) m_log->append( message );
}

void KDarLog::clear()
{
    m_log->clear();
}

void KDarLog::print( KPrinter* printer )
{
    if ( !printer ) return;
    QPainter p;
    p.begin( printer );

    QPaintDeviceMetrics metrics( printer );
    //copied basically from $QTDIR/examples/action/application.cpp
    int margin = 0;
    QRect body( margin, margin, metrics.width()-2*margin, metrics.height()-2*margin );
    QSimpleRichText richText( QStyleSheet::convertFromPlainText (m_log->text()), m_log->font(), m_log->context(), m_log->styleSheet(), m_log->mimeSourceFactory(), body.height() );
    richText.setWidth( &p, body.width() );
    QRect view( body );
    int page = 1;
    do
    {
        richText.draw( &p, body.left(), body.top(), view, colorGroup() );
        view.moveBy( 0, body.height() );
        p.translate( 0 , -body.height() );
        p.drawText( view.right() - p.fontMetrics().width( QString::number( page ) ), view.bottom() + p.fontMetrics().ascent() + 5, QString::number( page ) );
        if ( view.top()  >= richText.height() )
        {
            break;
        }
        printer->newPage();
        page++;
    } while (TRUE);
    p.end();
}

#include "kdarlog.moc"
