/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill   *
 *   jkerrb@shaw.ca   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdarmanager.h"
#include "../kdarmanager/kdarmanageritem.h"

#include <kiconview.h>
#include <klocale.h>

#include <qdatetime.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpoint.h>
#include <qrect.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qwidget.h>

class ToolTip : public QToolTip
{
public:
    ToolTip( QWidget* parent, KDarManager* view )
      : QToolTip( parent ),
        m_view( view )
    {
    }

    virtual ~ToolTip() {};

    void maybeTip( const QPoint & pos )
    {
        QListViewItem* qItem = m_view->itemAt( pos );
        if ( !qItem )
            return;
        KDarManagerItem* kdarItem = static_cast<KDarManagerItem*>( qItem );
        if ( !kdarItem )
            return;
        tip( m_view->itemRect( qItem ), m_view->tooltip( kdarItem, 0 ) );
    }

private:
    KDarManager *m_view;
};

KDarManager::KDarManager( QWidget* parent, const char * name )
    : KListView( parent, name )
{
    //Set up the KListView:
    addColumn( i18n( "Archive" ) );
    addColumn( i18n( "Root" ) );
    addColumn( i18n( "Date" ) );
    addColumn( i18n( "Status" ) );
    setMouseTracking( true );
    setAcceptDrops( true );
    setSelectionModeExt( KListView::FileManager );
    setAllColumnsShowFocus( true );
    setShowSortIndicator( true );
    setRootIsDecorated( true );
    setResizeMode( KListView::LastColumn );
    setShowToolTips( false );

    m_archiveInfo = new ToolTip( viewport(), this );
}


KDarManager::~KDarManager()
{
    if ( m_archiveInfo )
        delete m_archiveInfo;
}

QString KDarManager::tooltip( QListViewItem* item, int ) const
{
    KDarManagerItem* toolItem = static_cast<KDarManagerItem*>( item );
    if ( toolItem )
    {
        QString info;
        info += i18n( "<b>Name</b>: %1<br />" ).arg( toolItem->name() );
        info += i18n( "<b>Filename</b>: %1<br />" ).arg( toolItem->path() );
        info += i18n( "<b>Root</b>: %1<br />" ).arg( toolItem->root() );
        info += i18n( "<b>Date</b>: %1<br />" ).arg( toolItem->date().toString() );
        info += i18n( "<b>Compression:</b> %1<br />").arg( toolItem->compressionAlgorithmName() );
        info += i18n( "<b>Encrypted</b>: %1<br />" ).arg( QString( toolItem->isEncrypted() ? i18n( "yes" ) : i18n( "no" ) ) );
        info += i18n( "<b>Root EA</b> : %1<br />" ).arg( QString( toolItem->isRootEASaved() ? i18n( "saved" ) : i18n( "not saved" ) ) );
        info += i18n( "<b>User EA</b>: %1<br />" ).arg( QString( toolItem->isUserEASaved() ? i18n( "saved" ) : i18n( "not saved" ) ) );
        info += i18n( "<b>Status:</b> %1<br />").arg( toolItem->status() );
        info += i18n( "<b>Notes:</b> %1").arg( toolItem->notes() );
        return info;
    }
    return QString::null;
}

