/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "archivemanager.h"
#include "kdarmanageritem.h"
#include "kdararchive.h"

#include "../kcm_modules/kdarsettings.h"

#include "../kdardockwidgets/kdarmanager.h"

#include <qdatetime.h>

#include <iostream>

#include <dar/macro_tools.hpp>

ArchiveManager::ArchiveManager( KDarManager* view )
    : m_view( view )
{
    m_database = 0;
    //Disable sorting on the view
    if ( m_view )
        m_view->setSortColumn( -1 );
    //Load the list of managed archives
    loadManagedArchives();
}

ArchiveManager::~ArchiveManager()
{
    if ( m_database )
        delete m_database;
}

bool ArchiveManager::addDatabase( QString path )
{
    QString database( path + "." + DBEXTENSION );
    QString name( path );
    libdar::database* db;
    try
    {
        db = new libdar::database();
        if ( db )
        {
            db->dump( m_info, database, true );
            delete db;
        }
        if ( m_view )
        {
            //Add the database to the manager's view
            (void) new KDarManagerItem( m_view, name, "", "", "", "", "", "" );
        }
    }
    catch ( ... )
    {
        std::cout << "exception caught! ArchiveManager::addDatabase()" << std::endl;
        if ( db )
            delete db;
    }
}

bool ArchiveManager::addArchive( QString name, QString root, QString path, QString status, QString notes )
{
    if ( m_view )
    {
        QString database( KDarArchive::pathBasename( path ) + "." + DBEXTENSION );
        //This creates a database for the top-level (full) archive
        QString archive( KDarArchive::pathBasename( path ) );
        try
        {
            m_database = new libdar::database();
//            libdar::catalogue* cat = libdar::macro_tools_get_catalogue_from( m_info, path, EXTENSION, libdar::crypto_none, "", 0 );
//            m_database->add_archive( *cat, KDarArchive::pathBasename( path ), KDarArchive::basename( path ) );
            m_database->dump( m_info, database, true );

            //m_manager->op_add( database, archive, "", false );
            //add an archive to the managed list
//            addItem( name, root, QString::null, path, status, notes, QString::null, database );
        }
        catch ( libdar::Egeneric & e )
        {
            std::cout << "unable to create manager database." << std::endl;
            std::cout << "made it here!!!!" << std::endl;
            std::cout << "message: " << e.get_message() << std::endl;
            //Save the new list of managed archives
            saveManagedArchives();
            return false;
        }
        //Save the new list of managed archives
        saveManagedArchives();
        return true;
    }
    else
        return false;
}

bool ArchiveManager::addArchive( QString name, QString root, QString path, KDarManagerItem* parent, QString status, QString notes )
{
    if ( !parent )
        return false;
//    if ( m_view )
//    {
 //       //add an archive to the managed list
 //       //We are using one database per top-level (non-differential) archive
 //       //This creates a database for the top-level (full) archive
//        QString archive( KDarArchive::pathBasename( path ) );
 //       try
 //       {
 //           m_manager->op_add( parent->database(), archive, parent->pathBasename() , false );
 //           addItem( name, root, QString::null, path, status, notes, parent->name(), parent->database() );
 //           //Save the new list of managed archives
 //       }
 //       catch ( ... )
 //       {
 //           std::cout << "unable to add archive to manager database." << std::endl;
 //           saveManagedArchives();
 //           return false;
//        }
 //       saveManagedArchives();
 //       return true;
 //   }
 //   else
 //       return false;
    return false;
}

bool ArchiveManager::removeArchive( KDarManagerItem* archive, bool saveList )
{
    if ( archive )
    {
//        if ( !archive->parent() )
 //       {
 //           //This is a top-level archive which has it's own database.
 //           //So remove the database
 //           std::cout << "removing database file: " << archive->database() << std::endl;
 //           QFile::remove( archive->database() );
 //       }
 //       else
 //       {
 //           std::cout << "NOT removing database file: " << archive->database() << std::endl;
 //           //otherwise we remove the arhive from the manager
////            m_manager->op_del( archive->database(), nummin, nummax, false );
 //       }
        delete archive;
//        if ( saveList )
 //           saveManagedArchives();
 //       return true;
    }
    else
        return false;
}

QString ArchiveManager::archiveInfo( const QString & name )
{
    if ( !name.isEmpty() )
    {
        //Find the archive by iterating over the list's items
        if ( m_view )
        {
            QListViewItemIterator it( m_view );
            while ( it.current() )
            {
                KDarManagerItem* item = static_cast<KDarManagerItem*>( it.current() );
                if ( item->name() == name )
                {
                    QString info;
                    info += "\n";
                    info += "Name: " + item->name() + "\n";
                    info += "------------------------------------\n";
                    info += "Root: " + item->root() + "\n";
                    info += "Filename: " + item->path() + "\n";
                    info += "Date: " + item->date().toString() + "\n";
                    info += "DAR archive format: " + QString( item->isDarArchive() ? "yes" : "no" ) + "\n";
                    info += "Archive format version: " + item->darVersion() + "\n";
                    info += "Internal name: " + item->internalName() + "\n";
                    info += "Compression : " + item->compressionAlgorithmName() + "\n";
                    info += "Command line: " + item->commandLine() + "\n";
                    info += "Encrypted: " + QString( item->isEncrypted() ? "yes" : "no" ) + "\n";
                    info += "Root EA saved: " + QString( item->isRootEASaved() ? "yes" : "no" ) + "\n";
                    info += "User EA saved: " + QString( item->isUserEASaved() ? "yes" : "no" ) + "\n";
                    info += "basename: " + item->basename() + "\n";
                    info += "pathname: " + item->pathname() + "\n";
                    info += "pathBasename: " + item->pathBasename() + "\n";
                    info += "Notes: " + item->notes() + "\n";
                    return info;
                }
                ++it;
            }
        }
    }
    return QString::null;
}

QStringList ArchiveManager::managedArchives() const
{
    //return the list of managed archives:
    QStringList managed;
    QString sep = ",";
    if ( m_view )
    {
        QListViewItemIterator it( m_view );
        KDarManagerItem* item;
        while ( it.current() )
        {
            item = static_cast<KDarManagerItem*>( it.current() );
            if ( item )
            {
//                std::cout << "ArchiveManager::managedArchives(): name: " << item->name() << std::endl;
                managed << item->name() + sep + item->root() + sep + item->date().toString() + sep + item->status() + sep + item->notes() + sep + item->path() + sep + item->parentName() + sep + item->database();
            }
            ++it;
        }
        return managed;
    }
    else
        return QString::null;
}

void ArchiveManager::loadManagedArchives()
{
    //Display the archives in the view...
    if ( m_view )
    {
        m_view->clear();
        QStringList archives = KDarSettings::managedArchives();
        QString sep = ",";
        QString name;
        QString root;
        QString date;
        QString status;
        QString notes;
        QString path;
        QString parentName;
        QString database;
        //TODO do an intelligent sort algorithm...
        m_view->setSortColumn( -1 );
        for ( QStringList::Iterator it = archives.begin(); it != archives.end(); ++ it )
        {
            name = (*it).section( sep, 0, 0 );
            root = (*it).section( sep, 1, 1 );
            date = (*it).section( sep, 2, 2 );
            status = (*it).section( sep, 3, 3 );
            notes = (*it).section( sep, 4, 4 );
            path = (*it).section( sep, 5, 5 );
            parentName = (*it).section( sep, 6, 6 );
            database = (*it).section( sep, 7, 7 );
            addItem( name, root, date, path, status, notes, parentName, database );
        }
    }
}

void ArchiveManager::saveManagedArchives()
{
    KDarSettings::setManagedArchives( managedArchives() );
    KDarSettings::writeConfig();
}

void ArchiveManager::addItem( QString name, QString root, QString date, QString path, QString status, QString notes, QString parentName, QString database )
{
    if ( m_view )
    {
        //Does parent exit?
        KDarManagerItem* parent = 0;
        if ( !parentName.isNull() )
            parent = static_cast<KDarManagerItem*>( m_view->findItem( parentName, 0 ) );
        if ( parent )
        {
            //We need to add the archive as a last child. i.e. differential
            //archives can only be added as siblings of the most recent archive.
            KDarManagerItem* item = parent->firstChild();
            KDarManagerItem* previousItem = item;
            while ( item )
            {
                previousItem = item;
                item = item->nextSibling();
            }
            (void) new KDarManagerItem( parent, previousItem, name, root, date, status, notes, path, database );
        }
        else
        {
            KDarManagerItem* item = static_cast<KDarManagerItem*>( m_view->lastChild() );
            (void) new KDarManagerItem( m_view, item, name, root, date, status, notes, path, database );
        }
    }
}

QStringList ArchiveManager::find( const QString & pattern )
{
    QStringList foundItems;
    return foundItems;
}

QStringList ArchiveManager::findGlobal( const QString & pattern )
{
    QStringList foundItems;
    return foundItems;
}

