/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _ARCHIVEMANAGER_H_
#define _ARCHIVEMANAGER_H_

#include "../controllers/kdarinteraction.h"

#include <klocale.h>

#include <qstring.h>
#include <qstringlist.h>

#include <dar/libdar.hpp>
#include <dar/database.hpp>

class KDarManager;
class KDarManagerItem;

const QString DBEXTENSION( "dmd" );

/**
 * The ArchiveManager class handles everything to do with
 * managing archives. The main feature is to make searching and
 * restoring files from multiple differential archives easy.
 */

class ArchiveManager
{
public:

    /** Constructor **/
    ArchiveManager( KDarManager* view );

    /** Destructor **/
    virtual ~ArchiveManager();

    /**
     * Add a database to the manager.
     */
    virtual bool addDatabase( QString path );

    /**
     * Add an archive to the manager.
     */
    virtual bool addArchive( QString name, QString root, QString path, QString status, QString notes );
    /**
     * Add a differential archive to the manager.
     */
    virtual bool addArchive( QString name, QString root, QString path, KDarManagerItem* parent, QString status, QString notes );
    /**
     * Remove an archive from the manager.
     */
    virtual bool removeArchive( KDarManagerItem* archive, bool saveList = true );
    /**
     * Get info about an archive.
     */
    virtual QString archiveInfo( const QString & name );
    /**
     * returns a list of managed archives.
     */
    virtual QStringList managedArchives() const;
    /**
     * Load the managed archives into the listview
     */
    virtual void loadManagedArchives();
    /**
     * Save the managed archives from the listview
     */
    virtual void saveManagedArchives();
    /** add an item to the listview **/
    virtual void addItem( QString name, QString root, QString date = QString::null, QString path = QString::null, QString status = QString::null, QString notes = QString::null, QString parentName  = QString::null, QString database = QString::null );
    /**
     * Find a filename pattern in a top-level archive and it's differentials
     */
    virtual QStringList find( const QString & pattern );
    /**
     * Find a filename pattern within all top-level archives and their diff's
     */
    virtual QStringList findGlobal( const QString & pattern );


private:

    /** the view used to display the managed archives... **/
    KDarManager* m_view;

    KDarInteraction m_info;

    libdar::database* m_database;
};

#endif // _ARCHIVEMANAGER_H_
