/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarmanageritem.h"

#include <klistview.h>
#include <klocale.h>

KDarManagerItem::KDarManagerItem( KListView* view, QString archiveName, QString archiveRoot, QString archiveDate, QString archiveStatus, QString archiveNotes, QString archivePath, QString database )
    : KListViewItem( view, archiveName, archiveRoot, QString::null, archiveStatus ),
      KDarArchive( archivePath ),
      m_path( archivePath ),
      m_database( 0 )
{
    m_parentName = QString::null;
    setDateText( archiveDate );
    setNotes( archiveNotes );
    setDatabase( database );
};

KDarManagerItem::KDarManagerItem( KListView* view, KDarManagerItem* after, QString archiveName, QString archiveRoot, QString archiveDate, QString archiveStatus, QString archiveNotes, QString archivePath, QString database )
    : KListViewItem( view, after, archiveName, archiveRoot, archiveDate, archiveStatus ),
      KDarArchive( archivePath ),
      m_path( archivePath ),
      m_database( 0 )
{
    m_parentName = QString::null;
    setDateText( archiveDate );
    setNotes( archiveNotes );
    setDatabase( database );
};

KDarManagerItem::KDarManagerItem( KDarManagerItem* parent, QString archiveName, QString archiveRoot, QString archiveDate, QString archiveStatus, QString archiveNotes, QString archivePath, QString database )
    : KListViewItem( parent, archiveName, archiveRoot, QString::null, archiveStatus ),
      KDarArchive( archivePath ),
      m_path( archivePath ),
      m_database( 0 )
{
    if ( parent )
        m_parentName = parent->text( 0 );
    else
        m_parentName = QString::null;
    setDateText( archiveDate );
    setNotes( archiveNotes );
    setDatabase( database );
};

KDarManagerItem::KDarManagerItem( KDarManagerItem* parent, KDarManagerItem* after, QString archiveName, QString archiveRoot, QString archiveDate, QString archiveStatus, QString archiveNotes, QString archivePath, QString database )
    : KListViewItem( parent, after, archiveName, archiveRoot, QString::null, archiveStatus ),
      KDarArchive( archivePath ),
      m_path( archivePath ),
      m_database( 0 )
{
    if ( parent )
        m_parentName = parent->text( 0 );
    else
        m_parentName = QString::null;
    setDateText( archiveDate );
    setNotes( archiveNotes );
    setDatabase( database );
};

KDarManagerItem::~KDarManagerItem()
{
}

QString KDarManagerItem::parentName()
{
    KDarManagerItem* parentItem = this->parent();
    if ( parentItem )
        return parentItem->text( 0 );
    else
        return QString::null;
}

QString KDarManagerItem::key( int column, bool ascending ) const
{
    //We need to sort children based on the corresponding creation time
/*
    if ( column == 0 && parent() ) {
        QString permissions = text( 1 );
        QString file = permissions.left( 1 );
        if ( file == "d" )
            return file;
        else
            return ( QString ) "f";
    } else {
        return KListViewItem::key( column, ascending );
    }
*/
    //for now let sorting happen normally
    return KListViewItem::key( column, ascending );
}

int KDarManagerItem::compare( KListViewItem *i, int column, bool ascending ) const
{
    KDarManagerItem* item = static_cast<KDarManagerItem*>( i );
    //TODO implement this in some fashion
    return KListViewItem::compare( item, column, ascending );
}

KDarManagerItem* KDarManagerItem::firstChild()
{
    return static_cast<KDarManagerItem*>( KListViewItem::firstChild() );
}

KDarManagerItem* KDarManagerItem::nextSibling()
{
    return static_cast<KDarManagerItem*>( KListViewItem::nextSibling() );
}

KDarManagerItem* KDarManagerItem::parent()
{
    return static_cast<KDarManagerItem*>( KListViewItem::parent() );
}

void KDarManagerItem::setDateText( const QString & dateString )
{
    QString dateFormat = i18n( "archive date format", "d MMM yyyy" );
    if ( !dateString.isEmpty() )
    {
        QDateTime tmpDate( QDateTime::fromString( dateString ) );
        if ( tmpDate.isValid() )
            setText( 2, tmpDate.toString( dateFormat ) );
        else if ( date().isValid() )
            setText( 2, date().toString( dateFormat ) );
    }
    else if ( date().isValid() )
        setText( 2, date().toString( dateFormat ) );
}

libdar::database* KDarManagerItem::libdardatabase() const
{
    return m_database;
}
