/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KDARMANAGERITEM_H_
#define _KDARMANAGERITEM_H_

#include "kdararchive.h"

#include <klistview.h>

#include <qstring.h>
#include <qwidget.h>

#include <dar/database.hpp>

/**
 * This class encapsulates archive items in the manager
 */

class KDarManagerItem : public KListViewItem, public KDarArchive
{

public:
    KDarManagerItem( KListView* view, QString archiveName, QString archiveRoot, QString archiveDate = QString::null, QString archiveStatus = QString::null, QString archiveNotes = QString::null, QString archivePath = QString::null, QString database = QString::null );
    KDarManagerItem( KListView* view, KDarManagerItem* after, QString archiveName, QString archiveRoot, QString archiveDate = QString::null, QString archiveStatus = QString::null, QString archiveNotes = QString::null, QString archivePath = QString::null, QString database = QString::null );
    KDarManagerItem( KDarManagerItem* parent, QString archiveName, QString archiveRoot, QString archiveDate = QString::null, QString archiveStatus = QString::null, QString archiveNotes = QString::null, QString archivePath = QString::null, QString database = QString::null );
    KDarManagerItem( KDarManagerItem* parent, KDarManagerItem* after, QString archiveName, QString archiveRoot, QString archiveDate = QString::null, QString archiveStatus = QString::null, QString archiveNotes = QString::null, QString archivePath = QString::null, QString database = QString::null );

    virtual ~KDarManagerItem();

    QString path() const { return m_path; };
    void setPath( const QString & archivePath ) { m_path = archivePath; };

    QString name() const { return text( 0 ); };
    QString root() const { return text( 1 ); };
    QString status() const { return text( 3 ); };

    QString parentName();

    //reimplement some sorting methods
    QString key( int column, bool ascending ) const;
    int compare( KListViewItem *i, int column, bool ascending ) const;

/*#if !defined( Q_NO_USING_KEYWORD )
    using KListViewItem::firstChild;
    using KListViewItem::nextSibling;
    using KListViewItem::parent;
#endif
*/
    KDarManagerItem* firstChild();
    KDarManagerItem* nextSibling();
    KDarManagerItem* parent();

    void setDateText( const QString& date );

    libdar::database* libdardatabase() const;

private:
    /** Full path to the archive **/
    QString m_path;
    /** Parent archive's name **/
    QString m_parentName;
    /** The libdar::database to which this item belongs. **/
    libdar::database* m_database;
};

#endif // _KDARMANAGERITEM_H_
