/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@sourceforge.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdaraboutdialog.h"
#include "kdarcapabilities.h"

#include "../controllers/controller.h"

#include <kaboutdata.h>
#include <kaboutdialog.h>
#include <kactivelabel.h>
#include <kled.h>
#include <kurl.h>
#include <kurllabel.h>

#include <qlabel.h>
#include <qstring.h>

#include <dar/libdar.hpp>

KDarAboutDialog::KDarAboutDialog( QWidget* parent, const char* name, const QString & program, const QString & version, const QString & copyrightName, const QString & copyrightYear, const KAboutData* aboutData, controller* control )
    : KAboutDialog( KAboutDialog::AbtAppStandard, "KDar", KDialogBase::Close, KDialogBase::Close, parent, name, true )
{
    setProduct( program, version, copyrightName, copyrightYear );

    //An about page, a la kdelibs/kdeui/kaboutapplication.cpp ...
    QString aboutText;
    if ( !aboutData->shortDescription().isEmpty() )
        aboutText += aboutData->shortDescription() + "\n";
    if ( !aboutData->otherText().isEmpty() )
        aboutText += "\n" + aboutData->otherText() + "\n";
    if ( !aboutData->copyrightStatement().isEmpty() )
        aboutText += "\n" + aboutData->copyrightStatement() + "\n";
    if ( !aboutText.isEmpty() )
    {
        KAboutContainer* aboutPage = addContainerPage( i18n( "&About" ) );
        QLabel* aboutLabel = new QLabel( aboutText, this );
        aboutPage->addWidget( aboutLabel );
        if ( !aboutData->homepage().isEmpty() )
        {
            KURLLabel *url = new KURLLabel();
            url->setText( aboutData->homepage() );
            url->setURL( aboutData->homepage() );
            aboutPage->addWidget( url );
            connect( url, SIGNAL( leftClickedURL( const QString & ) ), this, SLOT( openURLSlot( const QString & ) ) );
        }
    }
    if ( control )
        {
            //Add a "Capabilities" page
            //Libdar version:
            KAboutContainer* capabilitiesPage = addContainerPage( i18n( "&Capabilities" ), AlignLeft );
            KDarCapabilities* capabilities = new KDarCapabilities( capabilitiesPage, "kdar_capabilities" );
            QString darVersion = i18n( "DAR Library (using version %1)" ).arg( control->libdarVersion() );
            capabilities->darLibraryLabel->setText( darVersion );
            libdar::deci deci_bits = libdar::deci( control->integerSizeBits() );
            if ( deci_bits.computer() == libdar::infinint( 0 ) )
                capabilities->integerinfinitebitKLed->on();
            else if ( deci_bits.computer() == libdar::infinint( 32 ) )
                capabilities->integer32bitKLed->on();
            else if ( deci_bits.computer() == libdar::infinint( 64 ) )
                capabilities->integer64bitKLed->on();

            if ( control->handlesZCompression() )
                capabilities->libzKLed->on();
            else
                capabilities->libzKLed->off();
            if ( control->handlesBZ2Compression() )
                capabilities->libbz2KLed->on();
            else
                capabilities->libbz2KLed->off();
            if ( control->handlesStrongCrypto() )
                capabilities->cryptoKLed->on();
            else
                capabilities->cryptoKLed->off();
            if ( control->handlesEA() )
                capabilities->eaKLed->on();
            else
                capabilities->eaKLed->off();
            if ( control->handlesLargefiles() )
                capabilities->largefilesKLed->on();
            else
                capabilities->largefilesKLed->off();
            if ( control->handlesNoDump() )
                capabilities->nodumpKLed->on();
            else
                capabilities->nodumpKLed->off();
            if ( control->usesSpecialAlloc() )
                capabilities->specialMemAllocKLed->on();
            else
                capabilities->specialMemAllocKLed->off();
            if ( control->isThreadSafe() )
                capabilities->threadsafeKLed->on();
            else
                capabilities->threadsafeKLed->off();

            capabilitiesPage->addWidget( capabilities );
        }
        QValueList<KAboutPerson> authors = aboutData->authors();
        if ( !authors.empty() )
        {
            //Just like in kdelibs/kdeui/kaboutapplication...
            QString authorTitle;
            if ( authors.count() == 1 )
                authorTitle = i18n( "A&uthor" );
            else
                authorTitle = i18n( "A&uthors" );
            KAboutContainer* authorPage = addScrolledContainerPage( authorTitle );
            KActiveLabel* activeLabel = new KActiveLabel( authorPage );
            activeLabel->setText( i18n( "Please use <a href=\"%1\">%2</a> to report bugs, do not mail the authors directly.\n" ).arg( aboutData->bugAddress() ).arg( aboutData->bugAddress() ) );
            authorPage->addWidget( activeLabel );
            QValueListConstIterator<KAboutPerson> it = authors.begin();
            QValueListConstIterator<KAboutPerson> end = authors.end();
            for ( ; it != end; ++it )
            {
                authorPage->addPerson( (*it).name(), (*it).emailAddress(), (*it).webAddress(), (*it).task() );
            }
        }
        QValueList<KAboutPerson> people = aboutData->credits();
        if ( !people.empty() )
        {
            //a la kdelibs/kdeui/kaboutapplication.cpp...
            KAboutContainer* thanks = addScrolledContainerPage( i18n( "&Thanks To" ) );
            QValueListConstIterator<KAboutPerson> it = people.begin();
            QValueListConstIterator<KAboutPerson> end = people.end();
            for ( ; it != end; ++it )
            {
                thanks->addPerson( (*it).name(), (*it).emailAddress(), (*it).webAddress(), (*it).task() );
            }
        }
        addLicensePage( i18n( "&License Agreement" ), aboutData->license() );
        setInitialSize( QSize(400,1) );
}

KDarAboutDialog::~KDarAboutDialog()
{
}

#include "kdaraboutdialog.moc"
