/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdaraddarchivedialog.h"
#include "kdaraddarchivewidget.h"

#include "../kdarmanager/kdarmanageritem.h"

#include <klistview.h>
#include <klocale.h>

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qptrlist.h>
#include <qwidget.h>

KDarAddArchiveDialog::KDarAddArchiveDialog( QWidget* parent, const QString & caption )
    : KDialogBase( parent, "kdaraddarchivedialog", true, caption, KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, true )
{
    //From example in kde:kdialogbase
    m_widget = new KDarAddArchiveWidget( this );
    setMainWidget( m_widget );
    connect( m_widget->differential, SIGNAL( toggled( bool ) ), this, SLOT( slotShowReferenceList( bool ) ) );
    slotShowReferenceList( false );
    //Disable sorting on the reference archive list
    m_widget->parentArchivesList->setSorting( -1 );
}

KDarAddArchiveDialog::~KDarAddArchiveDialog()
{
}

void KDarAddArchiveDialog::slotShowReferenceList( bool show )
{
    if ( show )
    {
        m_widget->chooseReferenceLabel->show();
        m_widget->parentArchivesList->show();
    }
    else
    {
        m_widget->chooseReferenceLabel->hide();
        m_widget->parentArchivesList->hide();
    }
}

void KDarAddArchiveDialog::addReferenceArchive( KDarManagerItem* item )
{
    if ( item && item->parent() )
    {
        std::cout << "item is " << item->name() << std::endl;
        QListViewItem* parent = m_widget->parentArchivesList->findItem( item->parent()->name(), 0 );
        std::cout << "parent is " << parent->text( 0 ) << std::endl;
        (void) new KListViewItem( parent, item->name(), item->root(), item->simpleDate() );
    }
    else
        (void) new KListViewItem( m_widget->parentArchivesList, item->name(), item->root(), item->simpleDate() );
}

void KDarAddArchiveDialog::clearReferenceList()
{
    m_widget->parentArchivesList->clear();
}

KListViewItem* KDarAddArchiveDialog::referenceArchive()
{
    //This works since only one item can be selected at a time...
    QPtrList<QListViewItem> selectedItems = m_widget->parentArchivesList->selectedItems();
    return (KListViewItem*) selectedItems.first();
}

#include "kdaraddarchivedialog.moc"

